Package: tipse
Title: Tipping Point Analysis for Survival Endpoints
Version: 2.0
Authors@R: 
      c(person(given = "Ajmal", 
           family = "Oodally", 
           email = "ajmal.oodally@novartis.com", 
           role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-8143-5011")
           ),
    person(given = "Craig", 
           family = "Wang", 
           email = "craig.wang@novartis.com", 
           role = c("aut"), 
           comment = c(ORCID = "0000-0003-1804-2463")
           ),
    person(given = "Zheng", 
           family = "Li", 
           email = "zheng.li@novartis.com", 
           role = c("ctb"),
           comment = c(ORCID = "0000-0002-7655-0290")
           ))
Description: Implements tipping point sensitivity analysis for time-to-event endpoints under different missing data scenarios, as described in Oodally et al. (2025) <doi:10.48550/arXiv.2506.19988>. Supports both model-based and model-free imputation, multiple imputation workflows, plausibility assessment and visualizations. Enables robust assessment for regulatory and exploratory analyses.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
Imports: MASS, ggplot2, survival, dplyr, stats, utils, knitr, purrr,
        rmarkdown
VignetteBuilder: knitr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-12 12:17:33 UTC; oodalaj1
Author: Ajmal Oodally [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-8143-5011>),
  Craig Wang [aut] (ORCID: <https://orcid.org/0000-0003-1804-2463>),
  Zheng Li [ctb] (ORCID: <https://orcid.org/0000-0002-7655-0290>)
Maintainer: Ajmal Oodally <ajmal.oodally@novartis.com>
Repository: CRAN
Date/Publication: 2026-05-12 12:40:02 UTC
Built: R 4.6.0; ; 2026-05-12 15:44:19 UTC; unix

Package: tipse
Title: Tipping Point Analysis for Survival Endpoints
Version: 2.0
Authors@R: 
      c(person(given = "Ajmal", 
           family = "Oodally", 
           email = "ajmal.oodally@novartis.com", 
           role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-8143-5011")
           ),
    person(given = "Craig", 
           family = "Wang", 
           email = "craig.wang@novartis.com", 
           role = c("aut"), 
           comment = c(ORCID = "0000-0003-1804-2463")
           ),
    person(given = "Zheng", 
           family = "Li", 
           email = "zheng.li@novartis.com", 
           role = c("ctb"),
           comment = c(ORCID = "0000-0002-7655-0290")
           ))
Description: Implements tipping point sensitivity analysis for time-to-event endpoints under different missing data scenarios, as described in Oodally et al. (2025) <doi:10.48550/arXiv.2506.19988>. Supports both model-based and model-free imputation, multiple imputation workflows, plausibility assessment and visualizations. Enables robust assessment for regulatory and exploratory analyses.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
Imports: MASS, ggplot2, survival, dplyr, stats, utils, knitr, purrr,
        rmarkdown
VignetteBuilder: knitr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-12 12:17:33 UTC; oodalaj1
Author: Ajmal Oodally [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-8143-5011>),
  Craig Wang [aut] (ORCID: <https://orcid.org/0000-0003-1804-2463>),
  Zheng Li [ctb] (ORCID: <https://orcid.org/0000-0002-7655-0290>)
Maintainer: Ajmal Oodally <ajmal.oodally@novartis.com>
Repository: CRAN
Date/Publication: 2026-05-12 12:40:02 UTC
Built: R 4.5.2; ; 2026-05-12 14:41:32 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,tipse)
S3method(summary,tipse)
export(assess_plausibility)
export(impute_landmark)
export(impute_model)
export(impute_percentile)
export(pool_results)
export(tipping_point_model_based)
export(tipping_point_model_free)
import(dplyr)
import(ggplot2)
import(knitr)
import(rmarkdown)
importFrom(MASS,mvrnorm)
importFrom(purrr,map_dfr)
importFrom(stats,median)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,survfit)
importFrom(survival,survreg)
importFrom(utils,head)
importFrom(utils,tail)

#' Print method for plausibility assessment
#'
#' @param x An object of class "plausibility_assessment"
#' @param ... Further arguments passed to or from other methods.
#'
#' @keywords internal
#'
print.plausibility_assessment <- function(x, ...) {
  cat("\u2192 Clinical plausibility assessment (Oodally et al, 2025):\n\n")
  cat("  At the tipping point, ", x$assessment, "\n\n", sep = "")
  cat("  Please carefully assess the clinical plausibility in light of the imputation method and study context.\n")
  invisible(x)
}

#' Pooling results using Rubin's Rule
#'
#' @description
#' Pooling results from multiple imputations using Rubin's Rule
#'
#' @param dat a list of data.frames from multiple imputation using one alpha or kappa parameter
#' @param cox.fit a coxph object which is used to compute HRs for each imputed datasets
#' @param conf.level confidence level for the returned confidence interval, default to be 0.95.
#'
#' @details
#' The Rubin's rule is applied to the Cox PH model results across imputed datasets as:
#'
#' \enumerate{
#'   \item \emph{Compute pooled HR:}
#'   \deqn{\bar{HR}_\lambda = \exp\Bigg(\frac{1}{M} \sum_{m=1}^{M} \log(HR_m)\Bigg)}
#'
#'   \item \emph{Compute pooled variance:}
#'   \deqn{\bar{\sigma}_\lambda^2 = \frac{1}{M} \sum_{m=1}^{M} \sigma_m^2 +
#'   \frac{1 + \frac{1}{M}}{M-1} \sum_{m=1}^{M} \big(\log(HR_m) - \overline{\log(HR_\lambda)}\big)^2}
#'
#'   \item \emph{Compute CI:}
#'   \deqn{\bar{HR}_\lambda \times \exp\big(\pm t_{\alpha/2} \sqrt{\bar{\sigma}_\lambda^2}\big)}
#' }
#' @return a data.frame of pooled hazard ratio and confidence interval estimate using Rubin's Rule
#' @importFrom survival coxph
#' @importFrom stats var qt
#' @export
#'
#' @examples
#' cox_fit <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
#' imputed_list <- impute_percentile(
#'   dat        = codebreak200,
#'   reason     = "Early dropout",
#'   impute     = "docetaxel",
#'   percentile = 30,
#'   J          = 5,
#'   seed       = 1
#' )
#' pool_results(imputed_list, cox_fit)
pool_results <- function(dat, cox.fit, conf.level = 0.95) {
  # checks for parent function
  if (conf.level <= 0 | conf.level > 1) {
    stop("Argument 'conf.level' must be between 0 and 1.")
  }

  if (!inherits(cox.fit, "coxph")) {
    stop("Argument 'cox_fit' must be a valid cox model object, e.g. coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200).")
  }


  J <- length(dat) # number of imputations

  small_cox <- function(dat.j) {
    model_call <- cox.fit$call # Extract original call
    model_call$data <- quote(dat.j) # Change the data argument
    fit.cox <- eval(model_call) # Re-run the model
    return(data.frame(log_hr_est = fit.cox$coefficients[1], log_hr_var = fit.cox$var))
  }

  results <- do.call(rbind, lapply(dat, small_cox))

  var_w <- mean(results$log_hr_var)
  var_b <- var(results$log_hr_est)
  var_t <- var_w + var_b + var_b/J

  lambda <- (1 + 1/J) * var_b/var_t
  q <- qt((1 - conf.level) / 2, df = (J - 1)/lambda^2, lower.tail = F)
  # Rubin's rule from J imputations
  hr_pool <- exp(mean(results$log_hr_est))
  log_hr_var_pool <- var_t # mean(results$log_hr_var) + (J + 1) / J * var(results$log_hr_est)
  hr_pool_upperCI <- exp(mean(results$log_hr_est) + q * sqrt(log_hr_var_pool))
  hr_pool_lowerCI <- exp(mean(results$log_hr_est) - q * sqrt(log_hr_var_pool))

  return(data.frame(HR = hr_pool, log_HR_var = log_hr_var_pool, HR_upperCI = hr_pool_upperCI, HR_lowerCI = hr_pool_lowerCI))
}

library(testthat)
library(survival)

# Create multiple imputations (list of data.frames)
dat_list <- list(
  codebreak200,
  codebreak200[sample(1:nrow(codebreak200), nrow(codebreak200), replace = TRUE), ],
  codebreak200[sample(1:nrow(codebreak200), nrow(codebreak200), replace = TRUE), ]
)

# Fit initial Cox model
cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = codebreak200)

test_that("pool_results returns correct structure and values", {
  res <- pool_results(dat_list, cox_fit, conf.level = 0.95)

  expect_s3_class(res, "data.frame")
  expect_true(all(c("HR", "log_HR_var", "HR_upperCI", "HR_lowerCI") %in% names(res)))

  # HR should be positive
  expect_true(res$HR > 0)
  # Variance should be positive
  expect_true(res$log_HR_var > 0)
  # CI bounds should make sense
  expect_true(res$HR_upperCI > res$HR_lowerCI)
})

test_that("pool_results handles different confidence levels", {
  res_90 <- pool_results(dat_list, cox_fit, conf.level = 0.90)
  res_95 <- pool_results(dat_list, cox_fit, conf.level = 0.95)

  # Wider CI for 95% than 90%
  expect_true((res_95$HR_upperCI - res_95$HR_lowerCI) > (res_90$HR_upperCI - res_90$HR_lowerCI))
})

test_that("pool_results works with single imputation (J=1)", {
  res <- pool_results(list(codebreak200), cox_fit, conf.level = 0.95)

  expect_s3_class(res, "data.frame")
  expect_true(res$HR > 0)
})

test_that("pool_results computes pooled HR correctly (manual check)", {
  # Compute manually for comparison
  results <- lapply(dat_list, function(df) {
    fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = df)
    c(log_hr_est = fit$coefficients[1], log_hr_var = fit$var)
  })
  results_df <- do.call(rbind, results)

  hr_manual <- exp(mean(results_df[, 1]))
  res <- pool_results(dat_list, cox_fit)

  expect_equal(round(res$HR, 6), round(hr_manual, 6))
})

test_that("pool_results handles edge case with very small conf.level", {
  res <- pool_results(dat_list, cox_fit, conf.level = 0.01)
  expect_true(res$HR_upperCI > res$HR_lowerCI)
})

#' Model-free imputation via percentile sampling
#'
#' @description
#' randomly sample from the percentile of best or worst patients (ordered by their observed times regardless of event or censoring) who do not require imputation.
#'
#' @param dat data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)
#' @param reason a string specifying the censoring reasons which require imputation. It must be one of the reasons from variable CNSRRS.
#' @param impute a string specifying the treatment arm(s) which require imputation. It must be one of the arms from variable TRT01P, the first level of TRT01P is considered as the control arm.
#' @param percentile numeric between 1 and 100, indicating the best (or worst) percentile of subjects to sample from.
#' @param J numeric indicating number of imputations.
#' @param seed Integer. Random seed for reproducibility.
#'
#' @details
#' We define two sets of subjects to sample from depending on the `impute` argument:
#'
#' 1. **Worst percentile of observations from treatment arm**
#'    \eqn{ \forall i \in N \mid \min\{T_i, C_i\} \leq F_{\min\{T_i, C_i\}}^{-1}(\kappa) }.
#' This set includes all indices \eqn{ i } where the minimum of \eqn{ T_i } (event time) and \eqn{ C_i } (censoring time) is **less than or equal to** the \eqn{\kappa}-th percentile of its distribution.
#'
#' 2. **Best percentile of observations control arm**
#'    \eqn{ \forall i \in N \mid \min\{T_i, C_i\} \geq F_{\min\{T_i, C_i\}}^{-1}(\kappa) }.
#' This set includes all indices \eqn{ i } where the minimum of \eqn{ T_i } and \eqn{ C_i } is **greater than or equal to** the \eqn{\kappa}-th percentile of its distribution.
#'
#' where \eqn{ F(\cdot) } denotes the cumulative distribution function (CDF) of the observed times and \eqn{F^{-1}(\kappa) } is the inverse CDF (quantile function) at percentile \eqn{\kappa}.
#'
#' @return a list of data.frame from each imputation with imputed AVAL and EVENT, where original variables are kept as AVALo and EVENTo.
#' @importFrom purrr map_dfr
#' @import dplyr
#' @export
#'
#' @examples
#' impute_percentile(
#'   dat        = codebreak200,
#'   reason     = "Early dropout",
#'   impute     = "docetaxel",
#'   percentile = 30,
#'   J          = 5,
#'   seed       = 1
#' )
impute_percentile <- function(dat, reason, impute, percentile, J, seed = NULL) {

  # handle seed
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    runif(1)
  if (is.null(seed))
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  }

  # sanitize input data
  dat <- sanitize_dataframe(dat)

  # obtain treatment arm label
  control <- levels(dat[["TRT01P"]])[1]
  trt <- levels(dat[["TRT01P"]])[2]

  # message(paste("Note:", control, "was chosen to be the control arm as the first level of variable TRT01P."))

  # check impute in one of the arms
  if (length(impute) > 1) {
    stop("Imputation only in one arm is allowed", call. = FALSE)
  }

  if (!(impute %in% c(control, trt))) {
    stop("Argument 'impute' must be one of the arms provided in column TRT01P.")
  }

  # check percentile greater than 0
  if (percentile <= 0 | percentile > 100) {
    stop("Argument 'percentile' must be greater than 0 and less or equal to 100.")
  }

  ids <- which(dat$TRT01P == impute & dat$CNSRRS %in% reason) # pts in control who discontinued
  if (length(ids) == 0) {
    stop("No subject identified meeting imputation criteria.")
  }

  ######## creating a copy of event and censor that will be replaced by imputed values where applicable ######
  dat <- dat %>% mutate(EVENTo = EVENT, AVALo = AVAL)
  dat_imp <- dat[ids, ]

  # define sampling population: best times if control arm or worst times if treatment arm
  impute_is_control <- identical(impute, control)
  dat_sample_select <- dat[-ids, ] %>%
    arrange(if (impute_is_control) {
      desc(AVAL)
    } else {
      AVAL
    }) %>%
    mutate(percentile_label = row_number() / n() * 100) %>%
    # select smallest percentile from data if parameter 'percentile' is too small
    filter(percentile_label <= max(min(percentile_label), percentile))

  if (nrow(dat_sample_select) < 1) {
    stop("There is no population to sample from, please check tipping range parameter values.")
  }

  km_data <- list()

  generate_replicate <- function(df, rep_id, n_to_impute = length(ids)) {
    df %>%
      sample_n(size = n_to_impute, replace = TRUE) %>%
      mutate(rep_sample = rep_id) # Add a replicate ID
  }

  # Generate J replicates
  dat_sample_select_rep <- purrr::map_dfr(1:J, ~ generate_replicate(dat_sample_select, .x)) %>%
    rename(AVAL_to_impute = AVAL, EVENT_to_impute = EVENT)

  dat_imp <- dat_imp %>%
    slice(rep(1:n(), times = J)) %>%
    mutate(rep = rep(1:J, each = nrow(dat_imp))) %>% # replicate J times
    bind_cols(dat_sample_select_rep %>% select(AVAL_to_impute, EVENT_to_impute, rep_sample)) %>%
    mutate(
      AVAL = if_else(AVAL_to_impute >= AVAL, AVAL_to_impute, AVAL), # use imputed time only if it's longer, since observed follow-up cannot change
      EVENT = EVENT_to_impute
    ) # use imputed event

  for (j in 1:J) {
    km_data[[j]] <- rbind(
      dat[-ids, ] %>% mutate(impute = FALSE),
      dat_imp %>% filter(rep == j) %>% select(-c(rep, AVAL_to_impute, EVENT_to_impute, rep_sample)) %>% mutate(impute = TRUE)
    ) %>%
      dplyr::mutate(iter = j, percentile = percentile)
  }

  return(km_data)
}

library(testthat)
library(dplyr)
library(purrr)

test_that("impute_percentile returns a list of data.frames with correct structure", {
  res <- impute_percentile(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    percentile = 20, J = 3, seed = 123
  )

  expect_type(res, "list")
  expect_length(res, 3) # J = 3 imputations

  for (df in res) {
    expect_s3_class(df, "data.frame")
    expect_true(all(c("TRT01P", "AVAL", "EVENT", "CNSRRS", "AVALo", "EVENTo", "iter", "percentile") %in% names(df)))
  }
})

test_that("impute_percentile imputes values correctly (AVAL changes for imputed rows)", {
  res <- impute_percentile(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    percentile = 50, J = 2, seed = 123
  )

  for (df in res) {
    expect_true(any(df$AVAL[df$impute] != df$AVALo[df$impute]))
  }
})

test_that("impute_percentile throws error for invalid impute arm", {
  expect_error(
    impute_percentile(codebreak200,
      reason = "Early dropout", impute = "invalid_arm",
      percentile = 20, J = 2
    ),
    "Argument 'impute' must be one of the arms"
  )
})

test_that("impute_percentile warns when multiple arms passed for impute", {
  expect_error(
    impute_percentile(codebreak200,
      reason = "Early dropout", impute = c("control", "sotorasib"),
      percentile = 20, J = 2
    ),
    "Imputation only in one arm is allowed"
  )
})

test_that("impute_percentile throws error for invalid percentile", {
  expect_error(
    impute_percentile(codebreak200,
      reason = "Early dropout", impute = "sotorasib",
      percentile = 0, J = 2
    ),
    "Argument 'percentile' must be greater than 0 and less or equal to 100"
  )
  expect_error(
    impute_percentile(codebreak200,
      reason = "Early dropout", impute = "sotorasib",
      percentile = 150, J = 2
    ),
    "Argument 'percentile' must be greater than 0 and less or equal to 100"
  )
})

test_that("impute_percentile handles case with no matching reason (empty ids)", {
  expect_error(
    impute_percentile(codebreak200,
      reason = "Nonexistent reason", impute = "sotorasib",
      percentile = 20, J = 2
    ),
    "No subject identified meeting imputation criteria."
  )
})

test_that("impute_percentile respects seed for reproducibility", {
  res1 <- impute_percentile(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    percentile = 20, J = 2, seed = 123
  )
  res2 <- impute_percentile(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    percentile = 20, J = 2, seed = 123
  )

  expect_equal(res1, res2)
})

test_that("impute_percentile computes EVENT correctly after imputation", {
  res <- impute_percentile(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    percentile = 20, J = 2, seed = 123
  )

  for (df in res) {
    expect_true(all(df$EVENT %in% c(0, 1)))
  }
})

#' Summarize Tipping Point Results (ARD Format)
#'
#' Creates a concise, analysis-results dataset (ARD) from a tipping point
#' analysis. Identifies the tipping point parameter where the upper CL of the hazard ratio crosses 1
#' and summarizes key metrics.
#'
#' @param object A `tipse` object returned by \link{tipping_point_model_free} or
#'   \link{tipping_point_model_based}.
#' @param ... Additional arguments not used.
#'
#' @return A data frame summarizing:
#'   \itemize{
#'   \item \code{HR} - hazard ratio at that tipping point
#'   \item \code{CONFINT} - 95% CI at tipping point
#'     \item \code{METHOD} - sampling type used
#'     \item \code{ARMIMP} - arm imputed
#'     \item \code{TIPPT} - parameter where upper CL first crosses 1
#'     \item \code{TIPUNIT} - parameter meaning
#'     \item \code{DESC} - textual interpretation
#'   }
#' @export
#'
#' @examples
#' \donttest{
#' # Hazard deflation in the control arm
#' cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
#' result1 <- tipping_point_model_based(
#'   dat = codebreak200,
#'   reason = "Early dropout",
#'   impute = "docetaxel",
#'   imputation_model = "weibull",
#'   J = 10,
#'   tipping_range = seq(0.1, 1, by = 0.05),
#'   cox_fit = cox1
#' )
#'
#' summary(result1)
#'
#' # Imputation in both arms
#' result2 <- tipping_point_model_based(
#'   dat = codebreak200,
#'   reason = "Early dropout",
#'   impute = c("docetaxel", "sotorasib"),
#'   imputation_model = "weibull",
#'   J = 10,
#'   tipping_range = list(seq(0.1, 1, by = 0.2), seq(0.5, 1.5, by = 0.2)),
#'   cox_fit = cox1,
#'   verbose = TRUE,
#'   seed = 12345
#' )
#'
#' summary(result2)
#' }
summary.tipse <- function(object, ...) {
  #----------------------------#
  # Validation
  #----------------------------#
  sanitize_tipse(object)

  summary_results <- object$imputation_results
  km_data <- object$imputation_data
  dat <- object$original_data
  arm <- object$arm_to_impute
  method <- object$method_to_impute

  control <- levels(dat[["TRT01P"]])[1]
  trt <- levels(dat[["TRT01P"]])[2]

  param_cols <- paste0("parameter_", arm)
  #----------------------------#
  # Identify tipping point
  #----------------------------#
  tipping_param <- summary_results[summary_results$tipping_point,]

  if (method == "percentile sampling") {
    tipping_unit <- paste(ifelse(arm == control, "best", "worst"), "percentile")
  } else if (method == "landmark sampling") {
    tipping_unit <- paste("number of subjects", ifelse(arm == control, "extended censoring", "set as events"))
  } else if (method == "hazard multiplication") {
    tipping_unit <- "% hazard multiplication"
  } else {
    stop("Unsupported tipping point method: ", method)
  }

  if (nrow(tipping_param) == 0 || is.null(tipping_param)) {
    param_tipping <- NA
    hr_tipping <- NA
    ci_tipping <- NA
    interp <- "No tipping point detected (upper CL < 1 across all parameters)."
  } else {
    hr_tipping <- tipping_param$HR
    ci_tipping <- sprintf("(%.4f-%.4f)", tipping_param$HR_lowerCI, tipping_param$HR_upperCI)
    param_tipping <- tipping_param[,param_cols]
    if (method == "hazard multiplication") {
      if (is.null(dim(param_tipping))){
param_tipping <- param_tipping*100
      } else {
      param_tipping <- apply(param_tipping*100, 1, paste, collapse = ",")
      }
    }
    interp <- paste0(
      "Tipping point (upper CL \u2265 1) reached when imputing ", paste(arm, collapse = ","),
      " arm at ", param_tipping, " ", tipping_unit, "."
    )
  }

  #----------------------------#
  # Summarize table (ARD format)
  #----------------------------#
  summary_tbl <- data.frame(
    HR = round(hr_tipping, 4),
    CONFINT = ci_tipping,
    METHOD = method,
    ARMIMP = paste(arm, collapse = ","),
    TIPPT = param_tipping,
    TIPUNIT = tipping_unit,
    DESC = interp,
    stringsAsFactors = FALSE
  )

  return(summary_tbl)
}

#' Model-based imputation from parametric distributions
#'
#' @description
#' Impute data with Weibull or exponential distribution conditional on follow-up time
#'
#' @param dat data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)
#' @param reason a string specifying the censoring reasons which require imputation. It must be one of the reasons from variable CNSRRS.
#' @param impute a string specifying the treatment arm(s) which require imputation. It must be one of the arms from variable TRT01P, the first level of TRT01P is considered as the control arm.
#' @param imputation_model a string specifying the parametric distribution used for imputation, can be "Weibull" or "exponential".
#' @param alpha hazard inflation (if treatment arm is imputed) or deflation (if control arm is imputed) rate
#' @param J numeric indicating number of imputations.
#' @param seed Integer. Random seed for reproducibility.
#'
#' @details
#' First fit model based on the data without dropout. And then impute the survival outcome based on exponential or Weibull distribution for those who dropped out.
#'
#' @importFrom stats runif
#'
#' @return a list of data.frame from each imputation with imputed AVAL and EVENT, where original variables are kept as AVALo and EVENTo.
#' @export
#'
#' @examples
#' impute_model(
#'   dat               = codebreak200,
#'   reason            = "Early dropout",
#'   impute            = "docetaxel",
#'   imputation_model  = "weibull",
#'   alpha             = 0.7,
#'   J                 = 5,
#'   seed              = 1
#' )


impute_model <- function(dat, reason, impute, imputation_model = c("weibull", "exponential"), alpha, J, seed = NULL) {

  # handle seed
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    runif(1)
  if (is.null(seed))
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  }

  # sanitize input data
  dat <- sanitize_dataframe(dat)
  imputation_model <- match.arg(imputation_model)
  # obtain treatment arm label
  control <- levels(dat[["TRT01P"]])[1]
  trt <- levels(dat[["TRT01P"]])[2]

  # check impute in one of the arms
  if (length(impute) > 1) {
    stop("Imputation only in one arm is allowed", call. = FALSE)
  }

  if (!(impute %in% c(control, trt))) {
    stop("Argument 'impute' must be one of the arms provided in column TRT01P.")
  }

  ######## creating a copy of event and censor that will be replaced by imputed values where applicable ######
  dat <- fit_model(dat, reason = reason, impute = impute, imputation_model = imputation_model)
  dat_imp <- dat %>%
    filter(impute) %>%
    mutate(alpha = alpha)

  km_data <- list()

  dat_imp <- dat_imp %>%
    slice(rep(1:n(), times = J)) %>%
    mutate(rep = rep(1:J, each = nrow(dat_imp))) %>% # replicate J times
    mutate(U = runif(n(), min = cdf, max = 1))
  dat_imp <- dat_imp %>%
    mutate(t = b * ((-log(1 - U) - (1 - alpha) * (AVAL / b)^a) / alpha)^(1 / a)) %>%
    mutate(AVAL = ifelse(t <= MAXAVAL, t, MAXAVAL)) %>% # imputed time won't exceed data cutoff
    mutate(EVENT = (t <= MAXAVAL)) # EVENT: 1 - event and 0 - censored

  for (j in 1:J) {
    km_data[[j]] <- rbind(dat %>% filter(!impute) %>% select(-c(a, b, cdf)), dat_imp %>% filter(rep == j) %>% select(-c(a, b, cdf, alpha, rep, U, t))) %>%
      dplyr::mutate(iter = j, alpha = alpha)
  }


  return(km_data = km_data)
}

library(testthat)
library(dplyr)

test_that("impute_model returns a list of data.frames with correct structure", {
  res <- impute_model(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    imputation_model = "weibull", alpha = 0.5, J = 3, seed = 123
  )

  expect_type(res, "list")
  expect_length(res, 3) # J = 3 imputations

  # Check each element is a data.frame with expected columns
  for (df in res) {
    expect_s3_class(df, "data.frame")
    expect_true(all(c("TRT01P", "AVAL", "EVENT", "CNSRRS", "AVALo", "EVENTo", "iter", "alpha") %in% names(df)))
  }
})

test_that("impute_model imputes values correctly (AVAL changes for imputed rows)", {
  res <- impute_model(codebreak200,
    reason = "Early dropout", impute = "docetaxel",
    imputation_model = "weibull", alpha = 0.5, J = 2, seed = 123
  )

  # Original AVALo should differ from imputed AVAL for imputed rows
  imputed_rows <- which(codebreak200$TRT01P == "docetaxel" & codebreak200$CNSRRS == "Early dropout")
  for (df in res) {
    expect_true(any(df$AVAL[imputed_rows] != df$AVALo[imputed_rows]))
  }
})

test_that("impute_model throws error for invalid impute arm", {
  expect_error(
    impute_model(codebreak200,
      reason = "Early dropout", impute = "invalid_arm",
      imputation_model = "weibull", alpha = 0.5, J = 2
    ),
    "Argument 'impute' must be one of the arms"
  )
})

test_that("impute_model warns when multiple arms passed for impute", {
  expect_error(
    impute_model(codebreak200,
      reason = "Early dropout", impute = c("control", "sotorasib"),
      imputation_model = "weibull", alpha = 0.5, J = 2
    ),
    "Imputation only in one arm is allowed"
  )
})


test_that("impute_model respects seed for reproducibility", {
  res1 <- impute_model(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    imputation_model = "weibull", alpha = 0.5, J = 2, seed = 123
  )
  res2 <- impute_model(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    imputation_model = "weibull", alpha = 0.5, J = 2, seed = 123
  )

  expect_equal(res1, res2)
})

test_that("impute_model computes EVENT correctly (1 if t <= longest_control)", {
  res <- impute_model(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    imputation_model = "weibull", alpha = 0.5, J = 2, seed = 123
  )

  for (df in res) {
    expect_true(all(df$EVENT %in% c(TRUE, FALSE)))
  }
})

#' Average Kaplan-Meier Curves Across Multiple Imputed Datasets
#'
#' Takes a list of multiply imputed datasets corresponding to a single tipping point
#' parameter and pools the Kaplan-Meier survival curves for a given treatment arm.
#' Uses log(-log) transformation and Rubin's rules for pooling across imputations.
#'
#' @param km_data List of data frames, each containing one multiply imputed dataset for a tipping point.
#'   Each data frame must contain columns `AVAL`, `EVENT`, `TRT01P`, `iter`, and a tipping point parameter depending on the method.
#' @param arm Character string specifying the treatment arm to pool (must match `TRT01P` levels).
#' @param conf_level Numeric. Confidence level for CIs (default = 0.95).
#'
#' @importFrom stats var qnorm
#' @return A data frame with the following columns:
#' \describe{
#'   \item{time}{Time points of the KM curve.}
#'   \item{survival_comb}{Pooled survival probability at each time point.}
#'   \item{survival_lcl_comb}{Lower 95% confidence limit of pooled survival.}
#'   \item{survival_ucl_comb}{Upper 95% confidence limit of pooled survival.}
#'   \item{stderr}{Standard error of the pooled log-log transformed estimate.}
#' }
#' @keywords internal
average_km <- function(km_data, arm, conf_level = 0.95) {
  if (!is.list(km_data) || length(km_data) == 0) stop("'km_data' must be a non-empty list")

  km_surv_probs <- vector("list", length(km_data))

  par <- names(km_data[[1]])[ncol(km_data[[1]])] # get parameter name

  for (i in seq_along(km_data)) {
    df <- km_data[[i]]
    if (!all(c("AVAL", "EVENT", "TRT01P", "iter") %in% names(df))) {
      stop("Each km_data element must contain columns: AVAL, EVENT, TRT01P, iter")
    }

    # subset to the specified arm and iteration
    arm_data <- df[df$TRT01P == arm & df$iter == i, ]

    # fit Kaplan-Meier model
    km_fit <- survfit(Surv(AVAL, EVENT) ~ 1, data = arm_data)
    km_sum <- summary(km_fit)

    # log(-log) transformation
    valid_idx <- km_sum$surv > 0 & km_sum$surv < 1
    km_surv_probs[[i]] <- data.frame(
      time = km_sum$time[valid_idx],
      survival_cll = log(-log(km_sum$surv[valid_idx])),
      stderr_cll = sqrt((1 / log(km_sum$surv[valid_idx])^2) *
        ((km_sum$std.err[valid_idx]^2) / (km_sum$surv[valid_idx]^2))),
      imputation = i,
      parameter = unique(arm_data[[par]])
    )
  }

  combined_data <- do.call(rbind, km_surv_probs)

  # keep time points present in more than one imputation
  time_counts <- table(combined_data$time)
  valid_times <- as.numeric(names(time_counts[time_counts > 1]))
  combined_data <- combined_data[combined_data$time %in% valid_times, ]

  # pool estimates across imputations using Rubin's rules
  pooled_results <- do.call(rbind, lapply(split(combined_data, combined_data$time), function(group) {
    m <- length(unique(group$imputation))
    W <- mean(group$stderr_cll^2)
    B <- var(group$survival_cll)
    T_var <- W + (1 + 1 / m) * B
    pooled_estimate <- mean(group$survival_cll)
    pooled_stderr <- sqrt(T_var)
    data.frame(
      time = unique(group$time),
      estimate = pooled_estimate,
      stderr = pooled_stderr,
      parameter = unique(group$parameter), row.names = F
    )
  }))

  # back-transform to survival scale and compute confidence intervals
  zval <- qnorm(1 - (1 - conf_level) / 2)
  pooled_results$survival_comb <- exp(-exp(pooled_results$estimate))
  pooled_results$survival_lcl_comb <- exp(-exp(pooled_results$estimate + zval * pooled_results$stderr))
  pooled_results$survival_ucl_comb <- exp(-exp(pooled_results$estimate - zval * pooled_results$stderr))

  return(pooled_results[, c("time", "parameter", "survival_comb", "survival_lcl_comb", "survival_ucl_comb", "stderr")])
}

library(testthat)
library(dplyr)

test_that("average_km throws error for invalid km_data input", {
  expect_error(
    average_km(NULL, arm = "A"),
    "'km_data' must be a non-empty list"
  )

  expect_error(
    average_km(list(), arm = "A"),
    "'km_data' must be a non-empty list"
  )
})

test_that("average_km throws error when required columns are missing", {
  bad_df <- data.frame(AVAL = 1:3, EVENT = c(1, 0, 1))
  expect_error(
    average_km(list(bad_df), arm = "A"),
    "Each km_data element must contain columns: AVAL, EVENT, TRT01P, iter"
  )
})

test_that("average_km returns expected structure", {
  # minimal reproducible KM dataset for two imputations
  df1 <- data.frame(
    AVAL = c(2, 4, 6),
    EVENT = c(1, 1, 1),
    TRT01P = "A",
    iter = 1,
    tipping_param = 10
  )

  df2 <- data.frame(
    AVAL = c(2, 4, 6), # same time points as df1
    EVENT = c(1, 1, 0),
    TRT01P = "A",
    iter = 2,
    tipping_param = 10
  )

  res <- average_km(km_data = list(df1, df2), arm = "A")

  expect_s3_class(res, "data.frame")
  expect_true(all(c(
    "time", "parameter", "survival_comb",
    "survival_lcl_comb", "survival_ucl_comb", "stderr"
  ) %in% names(res)))
})

test_that("average_km correctly filters by arm and iter", {
  df1 <- data.frame(
    AVAL = c(5, 8, 10),
    EVENT = c(1, 0, 0),
    TRT01P = c("A", "A", "B"), # arm B appears but should not be pooled
    iter = 1,
    tipping = 20
  )

  df2 <- data.frame(
    AVAL = c(5, 8),
    EVENT = c(1, 0),
    TRT01P = c("A", "A"),
    iter = 2,
    tipping = 20
  )

  res <- average_km(km_data = list(df1, df2), arm = "A")

  expect_true(all(res$parameter == 20))
})

test_that("average_km produces valid survival probabilities", {
  df1 <- data.frame(
    AVAL = c(2, 4, 6),
    EVENT = c(1, 1, 0),
    TRT01P = c("A", "A", "A"),
    iter = 1,
    tipping = 30
  )

  df2 <- data.frame(
    AVAL = c(2, 4, 6),
    EVENT = c(1, 0, 1),
    TRT01P = c("A", "A", "A"),
    iter = 2,
    tipping = 30
  )

  res <- average_km(km_data = list(df1, df2), arm = "A")

  expect_true(all(res$survival_comb >= 0 & res$survival_comb <= 1))
  expect_true(all(res$survival_lcl_comb >= 0))
  expect_true(all(res$survival_ucl_comb <= 1))
})

test_that("average_km throws an error when required columns are missing", {
  bad_df <- data.frame(
    AVAL = c(5, 6, 7),
    EVENT = c(1, 0, 1),
    # TRT01P missing
    iter = c(1, 1, 1),
    tipping = 50
  )

  expect_error(
    average_km(km_data = list(bad_df), arm = "A"),
    "must contain columns"
  )
})


test_that("average_km back transformation gives valid survival probabilities", {
  df1 <- data.frame(
    AVAL = c(2, 4, 6),
    EVENT = c(1, 1, 0),
    TRT01P = c("A", "A", "A"),
    iter = 1,
    tipping = 30
  )

  df2 <- data.frame(
    AVAL = c(2, 4, 6),
    EVENT = c(1, 0, 1),
    TRT01P = c("A", "A", "A"),
    iter = 2,
    tipping = 30
  )

  res <- average_km(km_data = list(df1, df2), arm = "A")

  expect_true(all(res$survival_comb > 0 & res$survival_comb < 1))
  expect_true(all(res$survival_lcl_comb > 0 & res$survival_lcl_comb < 1))
  expect_true(all(res$survival_ucl_comb > 0 & res$survival_ucl_comb < 1))

  # Lower ≤ estimate ≤ upper
  expect_true(all(res$survival_lcl_comb <= res$survival_comb))
  expect_true(all(res$survival_comb <= res$survival_ucl_comb))
})

#' Model-free imputation via landmark sampling
#'
#' @description
#' patients will be assigned deterministically an event time at the time of censoring  or extend the censoring time to the potential maximum follow-up of each patient.
#'
#' @param dat data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)
#' @param npts number of patients to be imputed
#' @param reason a string specifying the censoring reasons which require imputation. It must be one of the reasons from variable CNSRRS.
#' @param impute a string specifying the treatment arm(s) which require imputation. It must be one of the arms from variable TRT01P, the first level of TRT01P is considered as the control arm.
#' @param J numeric indicating number of imputations.
#' @param seed Integer. Random seed for reproducibility.
#'
#' @import dplyr
#' @details
#' patients will be assigned deterministically an event time at the time of censoring  or extend the censoring time to the potential maximum follow-up of each patient.

#' @return a list of data.frame from each imputation with imputed AVAL and EVENT, where original variables are kept as AVAL and EVENT.
#' @export
#'
#' @examples
#' impute_landmark(
#'   dat    = codebreak200,
#'   reason = "Early dropout",
#'   impute = "docetaxel",
#'   npts   = 5,
#'   J      = 5,
#'   seed   = 1
#' )
impute_landmark <- function(dat, reason, impute, npts, J, seed) {

  # handle seed
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    runif(1)
  if (is.null(seed))
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  }

  dat <- sanitize_dataframe(dat)
  # obtain treatment arm label
  control <- levels(dat[["TRT01P"]])[1]
  trt <- levels(dat[["TRT01P"]])[2]

  # check impute in one of the arms
  if (length(impute) > 1) {
    stop("Imputation only in one arm is allowed", call. = FALSE)
  }

  if (!(impute %in% c(control, trt))) {
    stop("Argument 'impute' must be one of the arms provided in column TRT01P.")
  }

  ids <- which(dat$TRT01P == impute & dat$CNSRRS %in% reason) # pts in control who discontinued
  if (length(ids) == 0) {
    stop("No subject identified meeting imputation criteria.")
  }

  # check ids
  if (npts < 0 | npts > length(ids)) {
    stop("Argument 'npts' must be greater than 0 and less or equal to the total number of patients meeting imputation criteria.")
  }

  ######## creating a copy of event and censor that will be replaced by imputed values where applicable ######
  dat <- dat %>% mutate(EVENTo = EVENT, AVALo = AVAL)
  dat_imp <- dat[ids, ]

  km_data <- list()

  npts_val <- npts
  impute_is_control <- identical(impute, control)
  impute_is_trt     <- identical(impute, trt)
  dat_imp <- dat_imp %>%
    slice(rep(1:n(), times = J)) %>%
    mutate(rep = rep(1:J, each = nrow(dat_imp))) %>% # replicate J times
    group_by(rep) %>%
    mutate(imp = (1:n() %in% sample(n(), size = npts_val, replace = FALSE))) %>%
    ungroup() %>%
    mutate(AVAL = ifelse(imp & impute_is_control, MAXAVAL, AVAL)) %>%
    mutate(EVENT = ifelse(imp & impute_is_trt, 1, 0))

  for (j in 1:J) km_data[[j]] <- rbind(dat[-ids, ] %>% mutate(impute = FALSE), dat_imp %>% filter(rep == j) %>% select(-c(rep, imp)) %>% mutate(impute = TRUE)) %>% mutate(iter = j, npts = npts_val)


  return(km_data = km_data)
}

library(testthat)
library(dplyr)

test_that("impute_landmark returns a list of data.frames with correct structure", {
  res <- impute_landmark(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    npts = 2, J = 3, seed = 123
  )

  expect_type(res, "list")
  expect_length(res, 3) # J = 3 imputations

  for (df in res) {
    expect_s3_class(df, "data.frame")
    expect_true(all(c("TRT01P", "AVAL", "EVENT", "CNSRRS", "AVALo", "EVENTo", "iter", "npts") %in% names(df)))
  }
})

test_that("impute_landmark imputes values correctly for treatment arm (EVENT=1 for imputed rows)", {
  res <- impute_landmark(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    npts = 2, J = 2, seed = 123
  )

  for (df in res) {
    imputed_rows <- df %>% filter(impute)
    expect_true(any(imputed_rows$EVENT == 1)) # EVENT should be 1 for imputed rows in trt arm
  }
})

test_that("impute_landmark imputes values correctly for control arm (AVAL extended to longest_control)", {
  res <- impute_landmark(codebreak200,
    reason = "Early dropout", impute = "docetaxel",
    npts = 2, J = 2, seed = 123
  )

  longest_control <- max(codebreak200$AVAL)
  for (df in res) {
    imputed_rows <- df %>% filter(impute)
    expect_true(any(imputed_rows$AVAL == longest_control)) # AVAL extended for control arm
  }
})

test_that("impute_landmark throws error for invalid impute arm", {
  expect_error(
    impute_landmark(codebreak200,
      reason = "Early dropout", impute = "invalid_arm",
      npts = 2, J = 2, seed = 123
    ),
    "Argument 'impute' must be one of the arms"
  )
})

test_that("impute_landmark warns when multiple arms passed for impute", {
  expect_error(
    impute_landmark(codebreak200,
      reason = "Early dropout", impute = c("docetaxel", "sotorasib"),
      npts = 2, J = 2, seed = 123
    ),
    "Imputation only in one arm is allowed"
  )
})

test_that("impute_landmark throws error for invalid npts", {
  expect_error(
    impute_landmark(codebreak200,
      reason = "Early dropout", impute = "sotorasib",
      npts = -1, J = 2, seed = 123
    ),
    "Argument 'npts' must be greater than 0"
  )
  expect_error(
    impute_landmark(codebreak200,
      reason = "Early dropout", impute = "sotorasib",
      npts = 100, J = 2, seed = 123
    ),
    "Argument 'npts' must be greater than 0"
  )
})

test_that("impute_landmark handles case with no matching reason (empty ids)", {
  expect_error(
    impute_landmark(codebreak200,
      reason = "Nonexistent reason", impute = "sotorasib",
      npts = 0, J = 2, seed = 123
    ),
    "No subject identified meeting imputation criteria."
  )
})

test_that("impute_landmark respects seed for reproducibility", {
  res1 <- impute_landmark(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    npts = 2, J = 2, seed = 123
  )
  res2 <- impute_landmark(codebreak200,
    reason = "Early dropout", impute = "sotorasib",
    npts = 2, J = 2, seed = 123
  )

  expect_equal(res1, res2)
})

#' Plot Tipping Point Analysis Results
#'
#' Visualizes hazard ratios and corresponding confidence intervals across tipping point
#' parameters from tipping point analyses (one-dimensional). Supports both model-free and model-based
#' approaches, including percentile sampling, landmark sampling, and hazard-based methods.
#' The tipping point—defined as the parameter value where the upper confidence interval
#' crosses 1—is highlighted on the plot.
#'
#' @param tipse An S3 object of class `"tipse"` returned from
#'   \code{tipping_point_model_free()} or \code{tipping_point_model_based()}.
#' @param title Optional character string specifying the plot title. Set to \code{NULL}
#'   to remove the title.
#' @param subtitle Optional character string specifying the plot subtitle. If \code{NULL},
#'   a default subtitle including the method and tipping point is used.
#' @param xlab Optional character string specifying the x-axis label. If \code{NULL},
#'   a method-specific default is used.
#' @param ylab Optional character string specifying the y-axis label. Defaults to
#'   \code{"Hazard Ratio"}.
#'
#' @return A \code{ggplot2} object showing hazard ratios and confidence intervals across
#'   tipping point parameters. The returned plot can be further modified using \code{+}.
#'
#' @keywords internal
#'
plot_tp <- function(tipse,
                    title = "Tipping Point Analysis",
                    subtitle = NULL,
                    xlab = NULL,
                    ylab = "Hazard Ratio") {
  #----------------------------#
  # Input validation
  #----------------------------#
  summary_results <- tipse$imputation_results
  dat <- tipse$original_data
  arm <- tipse$arm_to_impute
  method <- tipse$method_to_impute

  control <- levels(dat[["TRT01P"]])[1]
  trt <- levels(dat[["TRT01P"]])[2]
  param_cols <- paste0("parameter_", arm)

  # Determine default x-axis label
  x_label <- switch(method,
                    "percentile sampling" = if (arm == control) {
                      "% best event times sampled from"
                    } else {
                      "% worst event times sampled from"
                    },
                    "landmark sampling" = if (arm == control) {
                      sprintf("Number of %s subjects event-free at DCO", arm)
                    } else {
                      sprintf("Number of %s subjects as events at censoring time", arm)
                    },
                    "hazard multiplication" = sprintf("Hazard multiplication factor (%%) in %s arm", arm)
  )

  if (method == "hazard multiplication") {
    summary_results[param_cols] <- summary_results[param_cols] * 100
  }

  #----------------------------#
  # Identify tipping point
  #----------------------------#
  tipping_point <- summary_results[summary_results$tipping_point, param_cols]
    scale_x <- scale_x_continuous(
    breaks = seq(min(summary_results[param_cols]), max(summary_results[param_cols]), by = 10),
    labels = function(x) round(x)
  )

  if (is.null(subtitle)) {
    subtitle <- paste("Method:", method,
                      "| Tipping point (HR upper 95% CL \u2265 1):",
                      tipping_point)
  }
  #----------------------------#
  # Build ggplot
  #----------------------------#
  p <- ggplot(summary_results, aes(x = .data[[param_cols]], y = HR)) +
    geom_point(size = 2) +
    geom_errorbar(
      aes(ymin = HR_lowerCI, ymax = HR_upperCI),
      width = 0.1 # controls horizontal width of the CI bar ends
    ) +
    geom_hline(yintercept = 1, linetype = "dashed", color = "black") +
    geom_vline(xintercept = tipping_point, linetype = "dashed", color = "red") +
    labs(
      x = if (is.null(xlab)) x_label else xlab,
      y = ylab,
      title = title,
      subtitle = subtitle
    ) +
    scale_x +
    theme_minimal() +
    theme(
      plot.title = element_text(face = "bold"),
      plot.subtitle = element_text(color = "gray30"),
      plot.caption = element_text(hjust = 0),
      legend.position = "bottom",
      legend.title = element_text(hjust = 0.5)
    )


  return(p)
}

library(dplyr)
library(testthat)

# helper to create tipse object
make_tipse <- function(method, arm_to_impute, tipping_param) {
  cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = codebreak200)
  if (method %in% c("hazard multiplication")) {
    res <- tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = arm_to_impute,
      J = 2,
      tipping_range = tipping_param,
      cox_fit = cox_fit,
      seed = 123
    )
  } else {
    res <- tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = arm_to_impute,
      J = 2,
      tipping_range = tipping_param,
      cox_fit = cox_fit,
      method = method,
      seed = 123
    )
  }
}

test_that("plot_tp returns a ggplot object with minimal valid tipse input", {
  # --- use helper to build valid tipse object ---
  tipse_obj <- make_tipse("percentile sampling",
    "docetaxel",
    tipping_param = seq(5, 100, by = 10)
  )

  p <- plot_tp(tipse_obj)

  # --- class check ---
  expect_s3_class(p, "ggplot")

  # --- x-axis label check ---
  # percentile sampling + impute = control → % best event times
  expect_match(
    p$labels$x,
    "% best event times sampled from"
  )

  # --- required aesthetics ---
  expect_true("parameter_docetaxel" %in% names(p$data))
  expect_true("HR" %in% names(p$data))

  # ribbon (CI) must exist
  layer_geoms <- vapply(
    p$layers,
    function(l) class(l$geom)[1],
    character(1)
  )
  expect_true("GeomErrorbar" %in% layer_geoms)

  # point layer must exist
  expect_true("GeomPoint" %in% layer_geoms)

  # HR=1 reference line
  expect_true("GeomHline" %in% layer_geoms)

  # tipping point vertical line
  expect_true("GeomVline" %in% layer_geoms)

  # ensure tipping point line is at correct x value
  tp <- tipse_obj$imputation_results$parameter_docetaxel[
    tipse_obj$imputation_results$tipping_point
  ]
})

#' Patient level data from dummy trial
#'
#' Based on re-constructed Kaplan-Meier plot from ExteNET trial (Martin et al., 2017)
#'
#' @format
#' A data frame with 2840 rows and 5 columns:
#' \describe{
#'   \item{SUBJID}{Dummy patient ID}
#'   \item{TRT01P}{Treatment arm (Neratinib or placebo)}
#'   \item{AVAL}{iDFS time in days}
#'   \item{EVENT}{Indicator for iDFS event}
#'   \item{CNSRRS}{Censoring reason (Lost to follow-up or Other)}
#'   \item{MAXAVAL}{Maximum potential survival time, duration between randomization to data cut-off}
#' }
#' @source Martin, M., Holmes, F.A., Ejlertsen, B., Delaloge, S., Moy, B., Iwata, H., von Minckwitz, G., Chia, S.K., Mansi, J., Barrios, C.H. and Gnant, M., 2017. Neratinib after trastuzumab-based adjuvant therapy in HER2-positive breast cancer (ExteNET): 5-year analysis of a randomised, double-blind, placebo-controlled, phase 3 trial. The lancet oncology, 18(12), pp.1688-1700.
"extenet"

#' Plot Pooled Kaplan–Meier Curves from Tipping Point Analysis
#'
#' Visualizes averaged (pooled) Kaplan-Meier survival curves across multiple tipping
#' point parameters, highlighting the tipping point where the upper CL of the hazard ratio crosses 1.
#'
#' @param x An S3 object of class `"tipse"` returned from \link{tipping_point_model_free} or \link{tipping_point_model_based}.
#' @param type Type of plot, either "Kaplan-Meier" or "Tipping Point".
#' @param ... Additional arguments to specify title, subtitle, xlab and ylab.
#'
#' @return A ggplot2 object displaying pooled Kaplan–Meier curves.
#' @import ggplot2
#' @export
#'
#' @details
#' - If `type = Kaplan-Meier`, then the KM curves from multiply imputed datasets were pooled using Rubin’s rules
#' after complementary log-log transformation as described in Marshall et al. (2009).
#' it can be of interest to visually assess the scenario that tips the result and the shift it causes to the original KM curve,
#' although there is no objective measure to assess the robustness of the result.
#' - If `type = Tipping Point`, then the HR estimation across the range of tipping point parameters are plotted.
#'
#' @references Marshall, A., Altman, D.G., Holder, R.L. et al.
#' Combining estimates of interest in prognostic modelling studies after multiple imputation: current practice and guidelines.
#' BMC Med Res Methodol 9, 57 (2009). https://doi.org/10.1186/1471-2288-9-57
#'
#' @examples
#' \donttest{
#' cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
#' result <- tipping_point_model_based(
#'   dat = codebreak200,
#'   reason = "Early dropout",
#'   impute = "docetaxel",
#'   imputation_model = "weibull",
#'   J = 10,
#'   tipping_range = seq(0.1, 1, by = 0.05),
#'   cox_fit = cox1
#' )
#'
#' plot(result, type = "Kaplan-Meier")
#' plot(result, type = "Tipping Point")
#'
#'
#' # Imputation in both arms
#' result2 <- tipping_point_model_based(
#'   dat = codebreak200,
#'   reason = "Early dropout",
#'   impute = c("docetaxel", "sotorasib"),
#'   imputation_model = "weibull",
#'   J = 10,
#'   tipping_range = list(seq(0.4, 0.7, by = 0.1), seq(0.5, 1.5, by = 0.2)),
#'   cox_fit = cox1
#' )
#'
#' plot(result2, type = "Tipping Point")
#' }
#'
plot.tipse <- function(x, type = c("Kaplan-Meier", "Tipping Point"), ...) {
  # Validate class
  sanitize_tipse(x)

  type <- match.arg(type, c("Kaplan-Meier", "Tipping Point"))

  arm <- x$arm_to_impute
  method <- x$method_to_impute

  if (type == "Kaplan-Meier") {
    # Inform the user
    if (length(arm) == 2) {stop("Plotting Kaplan-Meier curves is not supported for imputation done in both arms.")}
    message("Plotting pooled Kaplan-Meier curves for arm '", arm, "' using method '", method, "' approach.")
    p <- plot_km(x, ...)
  } else if (type == "Tipping Point") {
    message("Plotting tipping point results for arm '", paste(arm, collapse = " & "), "' using method '", method, "' approach.")
    if (length(arm) == 1){
      p <- plot_tp(x, ...)
  } else if (length(arm == 2)){
    p <- plot_tp_2d(x, ...)
  }
  }
  return(p)
}

library(testthat)
library(dplyr)

# Cox model for testing
cox_fit <- survival::coxph(
  Surv(AVAL, EVENT) ~ TRT01P,
  ties = "exact",
  data = codebreak200
)

# ----------------------------------------------------------------------
# Create model-free tipse object for testing
# ----------------------------------------------------------------------
tp_free <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  J = 10,
  tipping_range = seq(5, 95, by = 5),
  cox_fit = cox_fit,
  method = "percentile sampling",
  seed = 123
)

# ----------------------------------------------------------------------
# Create landmark sampling tipse
# ----------------------------------------------------------------------
tp_det <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  J = 10,
  tipping_range = 1:20,
  cox_fit = cox_fit,
  method = "landmark sampling",
  seed = 123
)

# ----------------------------------------------------------------------
# Create model-based tipse object for testing
# ----------------------------------------------------------------------
tp_model <- tipping_point_model_based(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  imputation_model = "weibull",
  J = 10,
  tipping_range = seq(0.5, 1, by = 0.25),
  cox_fit = cox_fit,
  seed = 123
)

# ======================================================================
# TESTS
# ======================================================================

test_that("plot.tipse rejects non-tipse objects", {
  expect_error(plot.tipse(list(a = 1)), "tipse")
})


test_that("plot.tipse returns ggplot for Kaplan–Meier (percentile sampling)", {
  expect_message(
    p <- plot(tp_free, type = "Kaplan-Meier"),
    regexp = "Kaplan-Meier"
  )
  expect_s3_class(p, "ggplot")
})



test_that("plot.tipse works for landmark sampling", {
  # KM
  expect_message(
    p1 <- plot(tp_det, type = "Kaplan-Meier"),
    regexp = "landmark sampling"
  )
  expect_s3_class(p1, "ggplot")

  # TP
  expect_message(
    p2 <- plot(tp_det, type = "Tipping Point"),
    regexp = "landmark sampling"
  )
  expect_s3_class(p2, "ggplot")
})


test_that("plot.tipse works for model-based (hazard multiplication)", {
  # KM
  expect_message(
    p1 <- plot(tp_model, type = "Kaplan-Meier"),
    regexp = "hazard multiplication"
  )
  expect_s3_class(p1, "ggplot")

  # TP
  expect_message(
    p2 <- plot(tp_model, type = "Tipping Point"),
    regexp = "hazard multiplication"
  )
  expect_s3_class(p2, "ggplot")
})


test_that("plot.tipse enforces valid `type` argument", {
  expect_error(plot(tp_free, type = "NotAType"))
})


test_that("plot.tipse messages include correct arm and method", {
  expect_message(
    plot(tp_free, type = "Kaplan-Meier"),
    regexp = "docetaxel"
  )
  expect_message(
    plot(tp_det, type = "Kaplan-Meier"),
    regexp = "landmark sampling"
  )
})

#' Tipping Point Analysis (Model-Free)
#'
#' Performs a model-free tipping point analysis on time-to-event data by
#' repeatedly imputing censored observations under varying assumptions. The model-free
#' framework assumes that censored patients share similar survival
#' behavior with those from whom they are sampled, without fitting any parametric
#' survival model.
#'
#' @details
#' The **model-free tipping point analysis** provides a reproducible and intuitive
#' framework for exploring the robustness of treatment effects in time-to-event
#' (survival) endpoints when censoring may differ between study arms.
#'
#' Two sampling modes are supported:
#' \itemize{
#'   \item `method = "percentile sampling"` - performs re-sampling of event
#'         times from the best or worst percentile of observed patients ranked by their event or censoring time.
#'         The `tipping_range` specifies the percentiles of the observed data from which
#'         event times will be sampled to impute censored patients.
#'         For the treatment arm, use the worst percentiles (shortest survival times) from the
#'          observed data of both arms. For the control arm, use the best percentiles (longest survival times).
#'   \item `method = "landmark sampling"` - imputes a fixed number of censored
#'         patients deterministically. The `tipping_range` specifies the number of patients to be imputed.
#'         For the treatment arm, it defines the number of patients that will be assumed to
#'         have an event at their time of censoring. For the control arm, it defines the
#'         number of patients that will be assumed to be event-free at data cut-off, their maximum potential follow-up time.
#' }
#'
#' This function iteratively applies the percentile- or landmark-sampling
#' imputation procedure across a range of
#' tipping point parameters `tipping_range`. For each parameter value:
#' \enumerate{
#'   \item Multiple imputed datasets are generated (`J` replicates), where censored
#'         observations in the selected arm are replaced by sampled or reassigned
#'         event times according to the imputation method.
#'   \item A Cox proportional hazards model is fitted to each imputed dataset.
#'   \item Model estimates are pooled using **Rubin’s rules** to obtain a combined
#'         hazard ratio and confidence interval for that tipping point parameter.
#' }
#'
#' The process yields a series of results showing how the treatment effect changes
#' as increasingly conservative or optimistic assumptions are made about censored
#' observations. The *tipping point* is defined as the smallest value of the
#' sensitivity parameter (percentile or number of imputed patients) for which the upper
#' bound of the hazard ratio confidence interval crosses 1 - i.e., where the
#' apparent treatment benefit is lost.
#'
#' @param dat data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)

#' @param reason Vector specifying censoring reasons to be imputed.
#' @param impute a character vector specifying the arm(s) to impute. Can be one arm or both arms (a length-2 vector). Each value must be one of the arms from variable TRT01P. When both arms are supplied, imputation is applied independently for each arm.
#' @param J numeric indicating number of imputations.
#' @param tipping_range Numeric vector or when `length(impute) == 2` optionally a named or unnamed list of two numeric vectors (one per arm). When a list is supplied, all combinations of the two vectors are evaluated. Percentiles to use when `method = "percentile sampling"`. Number of patients to impute when `method = "landmark sampling"`.
#' @param verbose Logical. If `TRUE`, prints progress and analysis details.
#' @param method Character. Either `"percentile sampling"` or `"landmark sampling"`.
#' @param seed Integer, default as NULL. Random seed for reproducibility.
#' @param cox_fit A Cox model that will be used to calculate HRs on imputed datasets.
#'   In case of inclusion of stratification factors or covariates, conditional HR will be used.
#'
#' @return A `tipse` object containing:
#' \describe{
#'   \item{original data}{Input argument from 'data'.}
#'   \item{imputation_results}{A data frame of combined pooled model results across tipping points}
#'   \item{original_HR}{The original hazard ratio.}
#'   \item{reason_to_impute}{Input argument from 'reason'.}
#'   \item{arm_to_impute}{Input argument from 'impute'.}
#'   \item{method_to_impute}{Input argument from 'method'.}
#'   \item{imputation_data}{A list of imputed datasets for each tipping point value.}
#'   \item{seed}{Random seed.}
#' }
#' @import dplyr
#' @importFrom survival coxph
#' @importFrom utils head tail
#' @export
#'
#' @examples
#' cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
#' result <- tipping_point_model_free(
#'   dat = codebreak200,
#'   reason = "Early dropout",
#'   impute = "docetaxel",
#'   J = 10,
#'   tipping_range = seq(5, 95, by = 5),
#'   cox_fit = cox1,
#'   method = "percentile sampling"
#' )
#'
#' result2 <- tipping_point_model_free(
#'   dat = codebreak200,
#'   reason = "Early dropout",
#'   impute = "docetaxel",
#'   J = 10,
#'   tipping_range = seq(1, 21, by = 2),
#'   cox_fit = cox1,
#'   method = "landmark sampling"
#' )
tipping_point_model_free <- function(dat,
                                     reason,
                                     impute,
                                     J = 10,
                                     tipping_range = seq(5, 95, by = 5),
                                     cox_fit = NULL,
                                     verbose = FALSE,
                                     method = c("percentile sampling", "landmark sampling"),
                                     seed = NULL) {
  #----------------------------#
  # Setup and validation
  #----------------------------#
  method <- match.arg(method, c("percentile sampling", "landmark sampling"))

  # handle seed
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    runif(1)
  if (is.null(seed))
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  }

  dat <- sanitize_dataframe(dat)

  control <- levels(dat[["TRT01P"]])[1]
  trt <- levels(dat[["TRT01P"]])[2]

  if (!inherits(cox_fit, "coxph")) {
    stop("Argument 'cox_fit' must be a valid cox model object, e.g. coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200).")
  }
  if (length(impute) > 2 || !all(impute %in% c(control, trt))) {
    stop("Argument 'impute' must be one or both of the arms in column TRT01P.")
  }
  if (length(impute) == 2 && impute[1] == impute[2]) {
    stop("Argument 'impute' must not contain the same arm twice.")
  }

  if (length(reason) == 0) {
    stop("Argument 'reason' must specify at least one censoring reason to impute.")
  }

  # Normalise tipping_range: accept list (per-arm) or plain vector (shared)
  two_arms <- length(impute) == 2
  if (is.list(tipping_range)) {
    if (!two_arms) stop("A list 'tipping_range' is only allowed when two arms are supplied in 'impute'.")
    if (length(tipping_range) != 2) stop("When 'tipping_range' is a list it must have exactly two elements, one per arm.")
    range1 <- tipping_range[[1]]
    range2 <- tipping_range[[2]]
  } else {
    range1 <- tipping_range
    range2 <- tipping_range
  }

  if (method == "percentile sampling") {
    range1 <- sanitize_percentile_range(range1)
    if (two_arms) range2 <- sanitize_percentile_range(range2)
  } else {
    range1 <- sanitize_npts_range(dat, reason, impute[1], range1)
    if (two_arms) range2 <- sanitize_npts_range(dat, reason, impute[2], range2)
  }

  # column names for imputation results
  col1 <- paste0("parameter_", impute[1])
  col2 <- if (two_arms) paste0("parameter_", impute[2]) else NULL

  # Build the grid of parameter combinations to evaluate
  if (two_arms) {
    param_grid <- expand.grid(param1 = range1, param2 = range2)
  } else {
    param_grid <- data.frame(param1 = range1, param2 = NA_real_)
  }

  HR <- exp(cox_fit$coefficients[paste0("TRT01P", trt)])

  #----------------------------#
  # Print setup info
  #----------------------------#
  if (verbose) {
    cat("\u2192 Detected arms:\n")
    cat("   Control arm   :", control, "\n")
    cat("   Treatment arm :", trt, "\n")
    cat("   Imputing arm  :", paste(impute, collapse = " & "), "\n\n")
    cat("Starting tipping point analysis using method:", method, "\n")
    cat("Replicates per tipping point parameter:", J, "\n\n")
  }

  #----------------------------#
  # Main computation function
  #----------------------------#
  run_imputation <- function(param1, param2, cox_fit, verbose, dat, reason, impute, J, seed) {
    if (verbose) {
      fmt <- function(p) if (method == "percentile sampling") paste0(p, "%") else p
      label <- if (two_arms) paste(fmt(param1), "&", fmt(param2)) else fmt(param1)
      cat(" \u2192 Imputing for parameter:", label, "\n")
    }

    multiply_imputed_dfs <- switch(method,
      "percentile sampling" = impute_percentile(dat, reason, impute[1], percentile = param1, J = J, seed = seed),
      "landmark sampling"   = impute_landmark(dat, reason, impute[1], npts = param1, J = J, seed = seed)
    )
    if (two_arms) {
      multiply_imputed_dfs <- lapply(multiply_imputed_dfs, function(d) {
        switch(method,
          "percentile sampling" = impute_percentile(d, reason, impute[2], percentile = param2, J = 1, seed = seed)[[1]],
          "landmark sampling"   = impute_landmark(d, reason, impute[2], npts = param2, J = 1, seed = seed)[[1]]
        )
      })
    }

    pooled <- pool_results(multiply_imputed_dfs, cox_fit)
    pooled[[col1]] <- param1
    if (two_arms) pooled[[col2]] <- param2

    list(pooled = pooled, km_data = multiply_imputed_dfs)
  }

  #----------------------------#
  # Run all imputations
  #----------------------------#

  results <- mapply(
    run_imputation,
    param1 = param_grid$param1,
    param2 = param_grid$param2,
    MoreArgs = list(
      cox_fit = cox_fit, verbose = verbose,
      dat = dat, reason = reason, impute = impute, J = J, seed = seed
    ),
    SIMPLIFY = FALSE
  )

  summary_results <- dplyr::bind_rows(lapply(results, `[[`, "pooled"))
  km_data_list <- lapply(results, `[[`, "km_data")
  names(km_data_list) <- if (two_arms) {
    paste(param_grid$param1, param_grid$param2, sep = "|")
  } else {
    as.character(param_grid$param1)
  }

  #----------------------------#
  # Check tipping point
  #----------------------------#

  summary_results$tipping_point <- FALSE

  # for percentile/landmark sampling the crossing is always searched
  # in the increasing direction (larger parameter = more conservative)
  find_tip <- function(uci) {
    if (!any(uci < 1) || !any(uci > 1)) return(NA_integer_)
    idx <- if (method == "percentile sampling") tail(which(uci >= 1), 1) else head(which(uci >= 1), 1)
    if (abs(uci[idx] - 1) > 0.1)
      warning("Consider decreasing the step of 'tipping_range', the upper CL at tipping point was too far away from 1.0.")
    idx
  }

  if (!two_arms) {
    tip <- find_tip(summary_results$HR_upperCI)
    if (is.na(tip)) {
      warning("Tipping point not found, please check 'tipping_range'.")
    } else {
      summary_results$tipping_point[tip] <- TRUE
    }
  } else {
    # For each unique arm-2 parameter value (outer loop), find the tipping crossing
    # along the arm-1 axis (inner loop). Arm-1 varies within each arm-2 slice.
    any_found <- FALSE
    for (p2 in unique(summary_results[[col2]])) {
      slice_idx <- which(summary_results[[col2]] == p2)
      tip_in_slice <- find_tip(summary_results$HR_upperCI[slice_idx])
      if (!is.na(tip_in_slice)) {
        summary_results$tipping_point[slice_idx[tip_in_slice]] <- TRUE
        any_found <- TRUE
      }
    }
    if (!any_found) warning("Tipping point not found, please check 'tipping_range'.")
  }


  if (verbose) cat("\nTipping point analysis completed successfully \u2705 \n")

  #----------------------------#
  # Return structured output
  #----------------------------#

  tipse <- list(
    original_data = dat,
    original_HR = HR,
    reason_to_impute = reason,
    arm_to_impute = impute,
    method_to_impute = method,
    imputation_results = summary_results,
    imputation_data = km_data_list,
    seed = RNGstate
  )
  class(tipse) <- "tipse"
  return(tipse)
}

library(dplyr)
library(testthat)

# helper to create tipse object
make_tipse <- function(method, arm_to_impute, tipping_param) {
  cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = codebreak200)
  if (method %in% c("hazard multiplication")) {
    res <- tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = arm_to_impute,
      J = 2,
      tipping_range = tipping_param,
      cox_fit = cox_fit,
      seed = 123
    )
  } else {
    res <- tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = arm_to_impute,
      J = 2,
      tipping_range = tipping_param,
      cox_fit = cox_fit,
      method = method,
      seed = 123
    )
  }
}

test_that("plot_km works with model-free tipse object and draws expected layers", {
  # --- use helper to build valid tipse object ---
  tipse_obj <- make_tipse("percentile sampling",
    "docetaxel",
    tipping_param = seq(5, 100, by = 10)
  )
  p <- plot_km(tipse_obj)

  # Check ggplot object
  expect_s3_class(p, "ggplot")

  # Extract layers
  geoms <- vapply(p$layers, function(x) class(x$geom)[1], character(1))

  # There should be:
  #  - pooled curves (geom_step)
  #  - highlight curve (geom_step, red)
  #  - original curve (geom_step, orange)
  expect_true("GeomStep" %in% geoms)

  # Build plot for layer data inspection
  built <- ggplot_build(p)

  # Identify pooled KM curves: these come from first layer
  pooled_idx <- which(geoms == "GeomStep")[1]
  pooled_data <- built$data[[pooled_idx]]

  # Check x and y exist (instead of original column names)
  expect_true(all(c("x", "y") %in% names(pooled_data)))

  # Highlight curve (red)
  hl_idx <- which(geoms == "GeomStep")[2]
  hl_data <- built$data[[hl_idx]]

  expect_true(nrow(hl_data) > 0)

  # highlight line must be red
  expect_true(unique(hl_data$colour) == "red")


  # Original KM curve (orange dashed)
  orig_idx <- which(geoms == "GeomStep")[3]
  orig_data <- built$data[[orig_idx]]

  expect_true("surv" %in% names(orig_data) || "y" %in% names(orig_data))

  # Must be orange
  expect_true(unique(orig_data$colour) == "orange")
})

library(testthat)
library(survival)
library(dplyr)

# Cox model for testing
cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = codebreak200)

test_that("tipping_point_model_free returns correct structure for percentile sampling", {
  expect_warning(res <- tipping_point_model_free(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "sotorasib",
    J = 2,
    tipping_range = c(20, 40),
    cox_fit = cox_fit,
    method = "percentile sampling",
    seed = 123
  ), "Tipping point not found, please check 'tipping_range'.")

  expect_s3_class(res, "tipse")
  expect_true(all(c(
    "original_data", "original_HR", "reason_to_impute", "arm_to_impute",
    "method_to_impute", "imputation_results", "imputation_data"
  ) %in% names(res)))

  # Check imputation_results structure (single-arm uses arm-named parameter column)
  expect_true(all(c("HR", "HR_upperCI", "HR_lowerCI", "parameter_sotorasib", "tipping_point") %in% names(res$imputation_results)))
})


test_that("tipping_point_model_free throws error for invalid cox_fit", {
  expect_error(
    tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "sotorasib",
      J = 2,
      tipping_range = c(20, 40),
      cox_fit = "not_a_coxph",
      method = "percentile sampling"
    ),
    "Argument 'cox_fit' must be a valid cox model object"
  )
})

test_that("tipping_point_model_free throws error for invalid impute arm", {
  expect_error(
    tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "invalid_arm",
      J = 2,
      tipping_range = c(20, 40),
      cox_fit = cox_fit,
      method = "percentile sampling"
    ),
    "Argument 'impute' must be one or both of the arms"
  )
})

test_that("tipping_point_model_free throws error when reason is empty", {
  expect_error(
    tipping_point_model_free(
      dat = codebreak200,
      reason = character(0),
      impute = "sotorasib",
      J = 2,
      tipping_range = c(20, 40),
      cox_fit = cox_fit,
      method = "percentile sampling"
    ),
    "Argument 'reason' must specify at least one censoring reason"
  )
})

test_that("tipping_point_model_free throws error when percentile_range is missing for percentile sampling", {
  expect_error(
    tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "sotorasib",
      J = 2,
      tipping_range = NULL,
      cox_fit = cox_fit,
      method = "percentile sampling"
    ),
    "tipping_range"
  )
})

test_that("tipping_point_model_free throws error when npts_range is missing for landmark sampling", {
  expect_error(
    tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "sotorasib",
      J = 2,
      tipping_range = NULL,
      cox_fit = cox_fit,
      method = "landmark sampling"
    ),
    "tipping_range"
  )
})

test_that("tipping_point_model_free sets tipping_point flag correctly", {
  res <- tipping_point_model_free(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    J = 2,
    tipping_range = seq(10, 90, by = 10),
    cox_fit = cox_fit,
    method = "percentile sampling",
    seed = 123
  )

  expect_true("tipping_point" %in% names(res$imputation_results))
  expect_true(any(res$imputation_results$tipping_point %in% c(TRUE, FALSE)))
})

test_that("tipping_point_model_free respects seed for reproducibility", {
  res1 <- tipping_point_model_free(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    J = 2,
    tipping_range = seq(10, 90, by = 10),
    cox_fit = cox_fit,
    method = "percentile sampling",
    seed = 123
  )

  res2 <- tipping_point_model_free(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    J = 2,
    tipping_range = seq(10, 90, by = 10),
    cox_fit = cox_fit,
    method = "percentile sampling",
    seed = 123
  )

  expect_equal(res1$imputation_results, res2$imputation_results)
})

test_that("tipping_point_model_free two-arm percentile: correct structure and arm-named columns", {
  expect_warning(
    res <- tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = c("docetaxel", "sotorasib"),
      J = 2,
      tipping_range = c(20, 40),
      cox_fit = cox_fit,
      method = "percentile sampling",
      seed = 123
    ),
    "Tipping point not found"
  )

  expect_s3_class(res, "tipse")
  expect_equal(res$arm_to_impute, c("docetaxel", "sotorasib"))

  # arm-named parameter columns
  expect_true("parameter_docetaxel" %in% names(res$imputation_results))
  expect_true("parameter_sotorasib" %in% names(res$imputation_results))
  expect_false("parameter" %in% names(res$imputation_results))
  expect_false("parameter_1" %in% names(res$imputation_results))

  # Grid: 2 x 2 = 4 rows
  expect_equal(nrow(res$imputation_results), 4L)
})

test_that("tipping_point_model_free two-arm landmark: correct structure and arm-named columns", {
  expect_warning(
    res <- tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = c("docetaxel", "sotorasib"),
      J = 2,
      tipping_range = c(1, 2),
      cox_fit = cox_fit,
      method = "landmark sampling",
      seed = 123
    ),
    "Tipping point not found"
  )

  expect_s3_class(res, "tipse")
  expect_true("parameter_docetaxel" %in% names(res$imputation_results))
  expect_true("parameter_sotorasib" %in% names(res$imputation_results))
  expect_equal(nrow(res$imputation_results), 4L)
})

test_that("tipping_point_model_free two-arm: per-arm list tipping_range produces Cartesian grid", {
  expect_warning(
    res <- tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = c("docetaxel", "sotorasib"),
      J = 2,
      tipping_range = list(c(10, 20, 30), c(40, 50)),
      cox_fit = cox_fit,
      method = "percentile sampling",
      seed = 123
    ),
    "Tipping point not found"
  )

  # 3 x 2 = 6 rows
  expect_equal(nrow(res$imputation_results), 6L)
  expect_equal(length(unique(res$imputation_results$parameter_docetaxel)), 3L)
  expect_equal(length(unique(res$imputation_results$parameter_sotorasib)), 2L)
})

test_that("tipping_point_model_free two-arm: km_data_list names use pipe-separated parameters", {
  expect_warning(
    res <- tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = c("docetaxel", "sotorasib"),
      J = 2,
      tipping_range = c(20, 40),
      cox_fit = cox_fit,
      method = "percentile sampling",
      seed = 123
    ),
    "Tipping point not found"
  )

  expect_true(all(grepl("|", names(res$imputation_data), fixed = TRUE)))
})

test_that("tipping_point_model_free two-arm: errors on duplicate arms", {
  expect_error(
    tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = c("docetaxel", "docetaxel"),
      J = 2,
      tipping_range = c(20, 40),
      cox_fit = cox_fit,
      method = "percentile sampling"
    ),
    "must not contain the same arm twice"
  )
})

test_that("tipping_point_model_free two-arm: list tipping_range rejected for single arm", {
  expect_error(
    tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "docetaxel",
      J = 2,
      tipping_range = list(c(10, 20), c(30, 40)),
      cox_fit = cox_fit,
      method = "percentile sampling"
    ),
    "only allowed when two arms"
  )
})

test_that("tipping_point_model_free two-arm: reproducible with seed", {
  args <- list(
    dat = codebreak200, reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"),
    J = 2, tipping_range = c(20, 40), cox_fit = cox_fit,
    method = "percentile sampling", seed = 42
  )
  expect_warning(res1 <- do.call(tipping_point_model_free, args), "Tipping point not found")
  expect_warning(res2 <- do.call(tipping_point_model_free, args), "Tipping point not found")
  expect_equal(res1$imputation_results, res2$imputation_results)
})

library(testthat)
library(dplyr)

# helper to create tipse object
make_tipse <- function(dat, method, arm_to_impute, reason, tipping_param) {
  cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = dat)
  if (method == "hazard multiplication") {
    res <- tipping_point_model_based(
      dat = dat,
      reason = reason,
      impute = arm_to_impute,
      J = 2,
      tipping_range = tipping_param,
      cox_fit = cox_fit,
      seed = 123
    )
  } else {
    res <- tipping_point_model_free(
      dat = dat,
      reason = reason,
      impute = arm_to_impute,
      J = 2,
      tipping_range = tipping_param,
      cox_fit = cox_fit,
      method = method,
      seed = 123
    )
  }
}

test_that("assess_plausibility works for control arm with percentile sampling", {
  tipse <- make_tipse(codebreak200, "percentile sampling", "docetaxel", "Early dropout", tipping_param = seq(5, 100, by = 10))

  expect_output(
    assess_plausibility(tipse),
    regexp = "median duration of follow-up"
  )
})

test_that("assess_plausibility works for control arm with hazard multiplication (deflation)", {
  tipse <- suppressWarnings(make_tipse(codebreak200, "hazard multiplication", "docetaxel", "Early dropout", tipping_param = seq(0.1, 0.9, by = 0.2)))

  expect_output(
    assess_plausibility(tipse),
    regexp = "HR between imputed set in docetaxel arm"
  )
})

test_that("assess_plausibility works for treatment arm with landmark sampling", {
  tipse <- make_tipse(extenet, "landmark sampling", "neratinib", "Lost to follow-up", tipping_param = seq(1, 20, by = 1))

  expect_output(
    assess_plausibility(tipse),
    regexp = "event rate in imputed set in neratinib"
  )
})

test_that("assess_plausibility works for treatment arm with hazard multiplication (inflation)", {
  tipse <- make_tipse(extenet, "hazard multiplication", "neratinib", "Lost to follow-up", tipping_param = seq(1, 4, by = 0.5))

  expect_output(
    assess_plausibility(tipse),
    regexp = "HR between imputed set in neratinib arm"
  )
})

test_that("assess_plausibility errors for two-arm imputation", {
  cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = codebreak200)
  tipse_both <- suppressWarnings(tipping_point_model_based(
    dat = codebreak200, reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"),
    J = 2, tipping_range = seq(0.5, 0.9, by = 0.2),
    cox_fit = cox_fit, seed = 123
  ))
  expect_error(assess_plausibility(tipse_both), "both arms")
})

library(testthat)
library(dplyr)

# Cox model for testing
cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = codebreak200)
# tipse object
results <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  J = 2,
  tipping_range = seq(10, 80, by = 10),
  cox_fit = cox_fit,
  method = "percentile sampling",
  seed = 123
)

test_that("summary returns correct structure for percentile sampling", {
  res <- summary(results)

  expect_s3_class(res, "data.frame")
  expect_true(all(c("HR", "CONFINT", "METHOD", "ARMIMP", "TIPPT", "TIPUNIT", "DESC") %in% names(res)))
  expect_equal(res$METHOD, "percentile sampling")
  expect_true(grepl("percentile", res$TIPUNIT))
})


test_that("summary returns correct structure for landmark sampling", {
  # tipse object
  results2 <- tipping_point_model_free(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    J = 2,
    tipping_range = seq(1, 21, by = 5),
    cox_fit = cox_fit,
    method = "landmark sampling",
    seed = 123
  )
  res <- summary(results2)

  expect_equal(res$METHOD, "landmark sampling")
  expect_true(grepl("number of subjects", res$TIPUNIT))
})


test_that("summary handles hazard multiplication (inflation and deflation)", {
  results3 <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "sotorasib",
    J = 2,
    tipping_range = seq(1, 2, by = 0.05),
    cox_fit = cox_fit,
    seed = 123
  )
  res_infl <- summary(results3)
  expect_true(grepl("hazard multiplication", res_infl$TIPUNIT))

  results4 <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    J = 2,
    tipping_range = seq(0.1, 1, by = 0.05),
    cox_fit = cox_fit,
    seed = 123
  )
  res_defl <- summary(results4)
  expect_true(grepl("hazard multiplication", res_defl$TIPUNIT))
})

test_that("summary handles case with no tipping point detected", {
  expect_warning(
    results5 <- tipping_point_model_free(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "sotorasib",
      J = 2,
      tipping_range = c(20, 40),
      cox_fit = cox_fit,
      method = "percentile sampling",
      seed = 123
    ),
    "Tipping point not found"
  )
  res <- summary(results5)

  expect_true(is.na(res$TIPPT))
  expect_true(grepl("No tipping point detected", res$DESC))
})

test_that("summary formats CI and interpretation correctly", {
  res <- summary(results)

  expect_true(grepl("\\(", res$CONFINT))
  expect_true(grepl("Tipping point", res$DESC))
})

test_that("summary two-arm: returns one row per arm", {
  res_both <- suppressWarnings(tipping_point_model_based(
    dat = codebreak200, reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"),
    J = 2, tipping_range = seq(0.5, 0.9, by = 0.2),
    cox_fit = cox_fit, seed = 123
  ))
  res <- summary(res_both)

  expect_equal(nrow(res), 3L)
  expect_equal(unique(res$ARMIMP), c("docetaxel,sotorasib"))
  expect_true(all(c("HR", "CONFINT", "METHOD", "ARMIMP", "TIPPT", "TIPUNIT", "DESC") %in% names(res)))
  expect_true(all(res$METHOD == "hazard multiplication"))
})

#' Heatmap of Two-Dimensional Tipping Point Analysis
#'
#' Visualises the upper confidence limit of the hazard ratio across a Cartesian
#' grid of two imputation parameters (one per arm). Cells are shaded by
#' \code{HR_upperCL}, with the tipping line (where \code{HR_upperCI} crosses 1)
#' highlighted by a dashed contour line and flagged cells marked with a cross.
#'
#' @param tipse A \code{tipse} object returned by \link{tipping_point_model_based}
#'   or \link{tipping_point_model_free} with \code{impute} of length 2.
#' @param title Optional character string specifying the plot title. Set to \code{NULL}
#'   to remove the title.
#' @param subtitle Optional character string specifying the plot subtitle. If \code{NULL},
#'   a default subtitle including the method and tipping point is used.
#' @param xlab Optional character string specifying the x-axis label. If \code{NULL},
#'   a method-specific default is used.
#' @param ylab Optional character string specifying the y-axis label. Defaults to
#'   \code{"Hazard Ratio"}.
#' @param cross Optional logical to specifying if tipping points will be marked with a cross.
#' @param line Optional logical to specifying if tipping line will be marked with a contour.
#' @return A \code{ggplot2} object.
#' @keywords internal
#'
plot_tp_2d <- function(tipse,
                       title = "Two-Dimensional Tipping Point Analysis",
                       subtitle = NULL,
                       xlab = NULL,
                       ylab = NULL,
                       cross = TRUE,
                       line = TRUE) {
  #----------------------------#
  # Input validation
  #----------------------------#
  sanitize_tipse(tipse)

  arms <- tipse$arm_to_impute
  if (length(arms) != 2L) {
    stop("plot_tp_2d() requires a two-arm tipse object (length(impute) == 2).")
  }

  sr     <- tipse$imputation_results
  method <- tipse$method_to_impute

  col1 <- paste0("parameter_", arms[1])
  col2 <- paste0("parameter_", arms[2])

  #----------------------------#
  # Axis labels
  #----------------------------#
  dat     <- tipse$original_data
  control <- levels(dat[["TRT01P"]])[1]

  axis_label <- function(arm) {
    if (method == "percentile sampling") {
      paste0(ifelse(arm == control, "% best", "% worst"), " event times (", arm, ")")
    } else if (method == "landmark sampling") {
      paste0("No. subjects ", ifelse(arm == control, "extended censoring", "set as events"),
             " (", arm, ")")
    } else {
      # hazard multiplication: show % deviation from 1
      paste0("Hazard multiplication factor (", arm, ")")
    }
  }

  #----------------------------#
  # Scale parameters for display
  #----------------------------#
  plot_data <- sr

  x_label <- if (is.null(xlab)) axis_label(arms[1])
  y_label <- if (is.null(ylab)) axis_label(arms[2])

  if (is.null(subtitle) & line) {
    subtitle <- paste("Method:", method,
                      "| Dashed line marks tipping line (HR upper 95% CL \u2265 1)")
  } else if (is.null(subtitle)){
    subtitle <- paste("Method:", method)
  }

  #----------------------------#
  # Build heatmap
  #----------------------------#
  p <- ggplot(
    plot_data,
    aes(
      x    = .data[[col1]],
      y    = .data[[col2]]
    )
  ) +
    geom_tile(aes(fill = HR_upperCI), colour = "white", linewidth = 0.4) +
    # diverging palette centred on 1 (the decision boundary)
    scale_fill_gradient2(
      low      = "#2166ac",
      mid      = "#f7f7f7",
      high     = "#d6604d",
      midpoint = 1,
      name     = "HR Upper 95% CL"
    ) +
    labs(
      x        = x_label,
      y        = y_label,
      title    = title,
      subtitle = subtitle
    ) +
    theme_minimal(base_size = 11) +
    theme(
      plot.title = element_text(face = "bold"),
      plot.subtitle = element_text(color = "gray30"),
      plot.caption = element_text(hjust = 0),
      legend.position = "bottom",
      legend.title = element_text(hjust = 0.5)
    )

  # mark tipping-line cells
  if (cross) {p <- p + geom_point(
    data  = plot_data[plot_data$tipping_point, , drop = FALSE],
    shape = 4,   # cross
    size  = 3,
    colour = "black",
    stroke = 1.2
  )}
    # HR_upperCI = 1 contour
    if (line) {p <- p + geom_contour(
      aes(z = HR_upperCI),
      breaks   = 1,
      colour   = "black",
      linetype = "dashed",
      linewidth = 0.7
    )}

  return(p)
}

#' Assess Clinical Plausibility of Imputation Results
#'
#' This function facilitates the evaluation of clinical plausibility at the tipping point. It provides a text summary comparing event rates,
#' follow-up duration, or hazard ratios between treatment arms depending on the
#' imputation method and arm specified. NOTE: this function only supports imputation in one arm. 
#'
#' @param tipse A `tipse` object returned by one of \link{tipping_point_model_free} or \link{tipping_point_model_based}.
#' @param verbose Logical. If `TRUE`, prints assessment details.
#'
#' @return A character string summarizing the key information to facilitate clinical plausibility assessment based on
#' the imputation scenario.
#'
#' @import dplyr
#' @importFrom stats median
#' @importFrom survival coxph Surv
#' @export
#'
#' @examples
#' cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
#' result <- tipping_point_model_free(
#'   dat = codebreak200,
#'   reason = "Early dropout",
#'   impute = "docetaxel",
#'   cox_fit = cox1,
#'   method = "percentile sampling"
#' )
#'
#' assess_plausibility(result)
#'
assess_plausibility <- function(tipse, verbose = TRUE) {
  sanitize_tipse(tipse)

  if (length(tipse$arm_to_impute) == 2) {
    stop("The tipping point analysis involved imputation in both arms, it is not supported by this function.")
  }
  summary_results <- tipse$imputation_results
  km_data <- tipse$imputation_data

  control <- levels(tipse$original_data[["TRT01P"]])[1]
  trt <- levels(tipse$original_data[["TRT01P"]])[2]

  if (!any(summary_results$tipping_point)) {
    stop("No tipping point found, cannot assess plausibility.")
  }

  # cat("\u2192 Clinical plausibility assessment (Oodally et al, 2025): \n")
  param_col_name <- paste0("parameter_", tipse$arm_to_impute)
  tipping_param_val <- summary_results %>% filter(tipping_point) %>% pull(param_col_name)
  imputed_data <- km_data[[as.character(tipping_param_val)]]

  assessment <- NULL
  metric <- NULL

  if (tipse$arm_to_impute == trt) { # imputation in treatment arm

    if (tipse$method %in% c("percentile sampling", "landmark sampling")) {
      # compare event rate between imputed treatment and control
      rate_control <- tipse$original_data %>%
        filter(TRT01P == control) %>%
        summarise(rate = mean(EVENT)) %>%
        pull(rate)
      rate_trt <- mean(unlist(lapply(imputed_data, function(x) {
        x %>%
          filter(impute) %>%
          summarise(rate = mean(EVENT)) %>%
          pull(rate)
      })))

      assessment <- paste0(
        "the event rate in imputed set in ", trt, " arm was ", round(rate_trt * 100, 2),
        ", compared to ", round(rate_control * 100, 2), " in the ", control, " arm."
      )

      metric <- list(rate_trt = rate_trt, rate_control = rate_control)
    } else {
      # compare HR between imputed treatment vs control

      HR_imputed_vs_nonImputed <- 1 + tipping_param_val # 1 + inflation

      newHR <- HR_imputed_vs_nonImputed * tipse$original_HR

      assessment <- paste0("the HR between imputed set in ", trt, " arm and ", control, " arm was approximately ", newHR, ".")

      metric <- list(HR = newHR)
    }
  } else { # imputation in control arm

    if (tipse$method %in% c("percentile sampling", "landmark sampling")) {
      # compare follow up between imputed control vs treatment

      fup_trt <- tipse$original_data %>%
        filter(TRT01P == trt) %>%
        summarise(fup = median(AVAL)) %>%
        pull(fup)
      fup_control <- mean(unlist(lapply(imputed_data, function(x) {
        x %>%
          filter(impute) %>%
          summarise(fup = median(AVAL)) %>%
          pull(fup)
      })))

      assessment <- paste0(
        "the median duration of follow-up in imputed set in ", control, " arm was ", round(fup_control, 2),
        ", compared to ", round(fup_trt, 2), " in the ", trt, " arm."
      )

      metric <- list(fup_control = fup_control, fup_trt = fup_trt)
    } else {
      HR_imputed_vs_nonImputed <- 1 - tipping_param_val # 1 - deflation

      newHR <- HR_imputed_vs_nonImputed / tipse$original_HR

      # compare HR between imputed control vs treatment
      assessment <- paste0("the HR between imputed set in ", control, " arm and ", trt, " arm was approximately ", round(newHR, 2), ".")

      metric <- list(HR = newHR)
    }
  }
  assessment <- list(
      assessment = assessment,
      metric = metric,
      arm_imputed = tipse$arm_to_impute,
      method = tipse$method
    )
  class(assessment) <- "plausibility_assessment"
  if (verbose == FALSE){
    return(assessment)
  } else {
    print(assessment)
  }
  # cat(paste0("  At the tipping point, ", assessment, "\n", "  Please carefully assess the clinical plausibility based on this comparison together with the imputation method used in your study context."))
}

#' @import knitr
#' @import rmarkdown
#'
#' @noRd
#'
utils::globalVariables(c(
  "tipping_point", "parameter", "TRT01P", "EVENT", "impute", "EVENT", "AVAL",
  "a", "b", "imp", "MAXAVAL", "cdf", "U", "percentile_label", "AVAL_to_impute", "EVENT_to_impute",
  "rep_sample", "time", "survival_comb", "HR", "HR_lowerCI", "HR_upperCI", "CNSRRS", "surv", "rate", "fup"
))

#' Patient level data from dummy trial
#'
#' Based on re-constructed Kaplan-Meier plot from CodeBreak 200 trial (de Langen et al., 2023)
#'
#' @format
#' A data frame with 345 rows and 5 columns:
#' \describe{
#'   \item{SUBJID}{Dummy patient ID}
#'   \item{TRT01P}{Treatment arm (Sotorasib or Docetaxel)}
#'   \item{AVAL}{PFS time in days}
#'   \item{EVENT}{Indicator for PFS event}
#'   \item{CNSRRS}{Censoring reason (Early dropout or Other)}
#'   \item{MAXAVAL}{Maximum potential survival time, duration between randomization to data cut-off}
#' }
#' @source De Langen, A.J., Johnson, M.L., Mazieres, J., Dingemans, A.M.C., Mountzios, G., Pless, M., Wolf, J., Schuler, M., Lena, H., Skoulidis, F. and Yoneshima, Y., 2023. Sotorasib versus docetaxel for previously treated non-small-cell lung cancer with KRASG12C mutation: a randomised, open-label, phase 3 trial. The Lancet, 401(10378), pp.733-746.
"codebreak200"

#' Fit parametric model for selected subjects
#'
#' @param dat data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)
#' @param reason a string specifying the censoring reasons which require imputation. It must be one of the reasons from column CNSRRS.
#' @param impute a string specifying the treatment arm(s) which require imputation. It must be one of the arms from column TRT01P.
#' @param imputation_model a string specifying the parametric distribution used for imputation, can be "Weibull" or "exponential".
#'
#' @return data.frame with flags and fitted model parameters to be used for imputation
#' @importFrom survival survreg
#' @import dplyr
#' @importFrom MASS mvrnorm
#'
#' @details
#' The data.frame contains original columns, plus the following columns appended:
#' \tabular{ll}{
#' AVAL4  \tab Placeholder column to keep imputed survival times \cr
#' EVENT4 \tab Placeholder column to keep imputed events \cr
#' impute \tab Flag indicating whether the subject was selected for imputation \cr
#' a      \tab Shape parameter, equal to 1 if exponential \cr
#' b      \tab Scale parameter \cr
#' cdf    \tab Cumulative distribution function \cr
#' ...    \tab Some temporary columns
#' }
#' @keywords internal
fit_model <- function(dat, reason, impute, imputation_model = c("weibull", "exponential")) {
  # sanitize input data
  dat <- sanitize_dataframe(dat)
  imputation_model <- match.arg(imputation_model)
  # obtain treatment arm label
  control <- levels(dat[["TRT01P"]])[1]
  trt <- levels(dat[["TRT01P"]])[2]

  # check impute is one of the arms
  if (length(impute) > 1) {
    stop("Imputation only in one arm is allowed", call. = FALSE)
  }

  if (!(impute %in% c(control, trt))) {
    stop("Argument 'impute' must be one of the arms provided in column TRT01P.")
  }

  # Identify Patients from two arms who were not treated due or discontinued treatment (at any time)

  ids <- which(dat$TRT01P == impute & dat$CNSRRS %in% reason) # pts in control who discontinued
  if (length(ids) == 0) {
    stop("No subject identified meeting imputation criteria.")
  }

  ###  Weibull parameters estimated based on control arm patients excluding ids subset)
  fit_control <- survreg(Surv(AVAL, EVENT) ~ 1, data = dat %>% filter(TRT01P == control), dist = imputation_model)

  ###  Weibull parameters estimated based on trt arm patients excluding ids subset)
  fit_trt <- survreg(Surv(AVAL, EVENT) ~ 1, data = dat %>% filter(TRT01P == trt), dist = imputation_model)


  ab_control <- MASS::mvrnorm(nrow(dat), fit_control$icoef, fit_control$var)
  ab_trt <- MASS::mvrnorm(nrow(dat), fit_trt$icoef, fit_trt$var)

  if (imputation_model == "exponential") {
    ab_control <- cbind(ab_control, 0)
    ab_trt <- cbind(ab_trt, 0)
  }


  a_control <- 1 / exp(ab_control[, 2]) # shape = 1 / exp(log(scale))
  b_control <- exp(ab_control[, 1]) # scale = exp(intercept)

  a_trt <- 1 / exp(ab_trt[, 2]) # shape
  b_trt <- exp(ab_trt[, 1]) # scale


  # In patients to be imputed, the cumulative probability of event for these patient
  dat <- dat %>%
    mutate(AVALo = AVAL, EVENTo = EVENT) %>% # creating a copy of event and censor that will be replaced by imputed values where applicable
    mutate(impute = row_number() %in% ids) %>%
    bind_cols(a_control = a_control, b_control = b_control, a_trt = a_trt, b_trt = b_trt) %>%
    mutate(a = ifelse(TRT01P == control, a_control, a_trt), b = ifelse(TRT01P == control, b_control, b_trt)) %>%
    mutate(cdf = ifelse(impute, 1 - exp(-(AVAL / b)^a), NA))

  return(dat)
}

library(testthat)
library(dplyr)
library(survival)

test_that("fit_model returns expected columns", {
  res <- fit_model(codebreak200, reason = "Early dropout", impute = "sotorasib", imputation_model = "weibull")
  expect_s3_class(res, "data.frame")
  expect_true(all(c("TRT01P", "AVAL", "EVENT", "CNSRRS", "AVALo", "EVENTo", "impute", "a", "b", "cdf") %in% names(res)))
})

test_that("fit_model flags imputed subjects correctly", {
  res <- fit_model(codebreak200, reason = "Early dropout", impute = "sotorasib", imputation_model = "weibull")
  flagged_ids <- which(codebreak200$TRT01P == "sotorasib" & codebreak200$CNSRRS == "Early dropout")
  expect_true(all(res$impute[flagged_ids]))
  expect_false(any(res$impute[!flagged_ids]))
})

test_that("fit_model computes correct cdf for imputed rows", {
  res <- fit_model(codebreak200, reason = "Early dropout", impute = "sotorasib", imputation_model = "weibull")
  imputed_rows <- which(res$impute)
  for (i in imputed_rows) {
    expected_cdf <- 1 - exp(-(res$AVAL[i] / res$b[i])^res$a[i])
    expect_equal(res$cdf[i], expected_cdf)
  }
})

test_that("fit_model throws error for invalid impute arm", {
  expect_error(
    fit_model(codebreak200, reason = "Early dropout", impute = "invalid_arm", imputation_model = "weibull"),
    "Argument 'impute' must be one of the arms"
  )
})

test_that("fit_model warns when multiple arms passed for impute", {
  expect_error(
    fit_model(codebreak200, reason = "Early dropout", impute = c("control", "sotorasib"), imputation_model = "weibull"),
    "Imputation only in one arm is allowed"
  )
})

test_that("fit_model works with exponential distribution", {
  res <- fit_model(codebreak200, reason = "Early dropout", impute = "sotorasib", imputation_model = "exponential")
  expect_true(all(res$a == 1)) # For exponential, shape parameter should be 1
  expect_true(all(res$b > 0))
})

test_that("fit_model handles case with no matching reason (empty ids)", {
  expect_error(
    fit_model(codebreak200, reason = "Nonexistent reason", impute = "sotorasib", imputation_model = "weibull"),
    "No subject identified meeting imputation criteria."
  )
})

test_that("fit_model computes positive parameters", {
  res <- fit_model(codebreak200, reason = "Early dropout", impute = "sotorasib", imputation_model = "weibull")
  expect_true(all(res$a > 0))
  expect_true(all(res$b > 0))
})

#' Plot Pooled Kaplan–Meier Curves from Model-Free Tipping Point Analysis
#'
#' Visualizes averaged (pooled) Kaplan-Meier survival curves across multiple tipping
#' point parameters, highlighting the tipping point where the upper CL of the hazard ratio crosses 1.
#'
#' @param tipse An S3 object of class `"tipse"` returned from \code{tipping_point_model_free()} or \code{tipping_point_model_based()}.
#'
#' @importFrom survival survfit Surv
#' @return A \code{ggplot2} object displaying pooled Kaplan-Meier curves, with:
#' \itemize{
#'   \item Colored lines - pooled KM curves for each tipping point parameter
#'   \item Red line - tipping point where HR upper CL crosses 1
#'   \item Orange dashed line - original (unimputed) KM curve
#' }
#' @keywords internal

plot_km <- function(tipse) {
  #----------------------------#
  # Sanitize and validate input
  #----------------------------#

  sanitize_tipse(tipse)
  summary_results <- tipse$imputation_results
  dat <- tipse$original_data
  arm <- tipse$arm_to_impute
  method <- tipse$method_to_impute
  km_data <- tipse$imputation_data

  # Identify control and treatment arms
  control <- levels(dat$TRT01P)[1]
  trt <- levels(dat$TRT01P)[2]
  param_cols <- paste0("parameter_", arm)

  #----------------------------#
  # Average KM curves
  #----------------------------#
  res_all_km <- dplyr::bind_rows(
    lapply(
      km_data,
      FUN = average_km,
      arm = arm
    )
  )

  #----------------------------#
  # Identify tipping point
  #----------------------------#

  key <- paste(arm, method, sep = "_")

  highlight_param <- summary_results[summary_results$tipping_point, param_cols]

  if (is.null(highlight_param)) {
    warning("Could not determine tipping point.")
  }

  if (key == paste(control, "hazard multiplication", sep = "_")) {
    res_all_km$parameter <- res_all_km$parameter * 100
    highlight_param <- highlight_param * 100
  }

  highlight_data <- res_all_km[res_all_km$parameter == highlight_param, , drop = FALSE]

  #----------------------------#
  # Original (unimputed) KM curve
  #----------------------------#
  survfit_original <- survfit(Surv(AVAL, EVENT) ~ 1, data = subset(dat, TRT01P == arm))
  survfit_original_data <- data.frame(
    time = survfit_original$time,
    surv = survfit_original$surv,
    parameter = 0
  )

  #----------------------------#
  # Plot
  #----------------------------#
  p <- ggplot(res_all_km, aes(x = time, y = survival_comb, color = parameter, group = parameter)) +
    geom_step(alpha = 0.7) +
    scale_color_gradient(
      low = "darkblue", high = "lightblue",
      name = switch(method,
        "percentile sampling" = ifelse(arm == control, paste("% best event times sampled from"),
          paste("% worst event times sampled from")
        ),
        "landmark sampling" = ifelse(arm == control, paste("Number of ", arm, "subjects event-free at DCO"),
          paste("Number of", arm, "subjects as events at censoring time")
        ),
        "hazard multiplication" = paste("Hazard multiplication factor (%) in", arm, "arm"),
      ),
      guide = guide_colorbar(title.position = "bottom", title.hjust = 0.5)
    ) +
    geom_step(
      data = highlight_data, aes(x = time, y = survival_comb),
      color = "red", linewidth = 1.2, linetype = "solid"
    ) +
    geom_step(
      data = survfit_original_data, aes(x = time, y = surv),
      color = "orange", linewidth = 1, linetype = "dashed"
    ) +
    labs(
      title = paste("Pooled Kaplan-Meier Curves:", arm, "arm"),
      subtitle = paste("Method:", method, "| Tipping point (HR upper 95% CL \u2265 1):", highlight_param),
      caption = paste("Original KM curve is shown in orange and KM curve at tipping point is shown in red."),
      x = "Time",
      y = "Survival Probability"
    ) +
    theme_minimal() +
    theme(
      plot.title = element_text(face = "bold"),
      plot.subtitle = element_text(color = "gray30"),
      plot.caption = element_text(hjust = 0),
      legend.position = "bottom",
      legend.title = element_text(hjust = 0.5)
    )

  return(p)
}

sanitize_dataframe <- function(df) {
  # Check required columns
  required_cols <- c("AVAL", "TRT01P", "EVENT", "CNSRRS", "MAXAVAL")
  missing_cols <- setdiff(required_cols, names(df))

  if (length(missing_cols) > 0) {
    stop(paste("Missing required columns:", paste(missing_cols, collapse = ", ")))
  }

  # Check AVAL is numeric
  if (!is.numeric(df$AVAL)) {
    stop("Column 'AVAL' must be numeric.")
  }

  # Check TRT01P is factor
  if (!is.factor(df$TRT01P)) {
    df$TRT01P <- as.factor(df$TRT01P)
    msg <- sprintf('Column "TRT01P" is not a factor, converting to factor with default level for control arm as "%s".', levels(df$TRT01P)[1])
    warning(msg, call. = FALSE)
  }

  # Check TRT01P is factor
  if (nlevels(df$TRT01P) != 2) {
    stop("Column 'TRT01P' must be a factor containing two levels", call. = FALSE)
  }

  # Check EVENT is binary (0/1)
  if (!all(df$EVENT %in% c(0, 1))) {
    stop("Column 'EVENT' must be binary (values 0 or 1).")
  }

  # Return sanitized dataframe
  return(df)
}

# Ensure percentile_range is a numeric vector
sanitize_percentile_range <- function(percentile_range) {
  if (!is.vector(percentile_range) || !is.numeric(percentile_range)) {
    stop("`tipping_range` must be a numeric vector.")
  }
  if (any(percentile_range <= 0 | percentile_range > 100)) {
    stop("All values in `tipping_range` must be between 0 and 100.")
  }
  return(percentile_range)
}

# Ensure npts_range is a numeric vector and does not exceed available patients
sanitize_npts_range <- function(dat, reason, impute, npts_range) {
  if (!is.vector(npts_range) || !is.numeric(npts_range)) {
    stop("`tipping_range` must be a numeric vector.")
  }
  n_available <- dat %>%
    subset(CNSRRS %in% reason & TRT01P == impute) %>%
    nrow()

  if (any(npts_range > n_available)) {
    stop(paste0("All values in `tipping_range` must be <= number of available patients (", n_available, ")."))
  }
  return(npts_range)
}

sanitize_tipse <- function(tipse) {
  if (!inherits(tipse, "tipse")) {
    stop("`tipse` must be an object of class 'tipse', returned by tipping_point_model_free() or tipping_point_model_based().")
  }

  # Extract basic elements
  if (!all(c("imputation_results", "imputation_data", "original_data", "arm_to_impute", "method_to_impute") %in% names(tipse))) {
    stop("The `tipse` object is missing required components: imputation_results, imputation_data, original_data, arm_to_impute, or method_to_impute.")
  }

  method <- tipse$method_to_impute
  valid_methods <- c("percentile sampling", "landmark sampling", "hazard multiplication")
  if (!method %in% valid_methods) {
    stop("Invalid method in `tipse$method_to_impute`. Must be one of: ", paste(valid_methods, collapse = ", "))
  }
}

# Ensure hazard_deflation_range is a numeric vector
sanitize_tipping_range <- function(range) {
  if (!is.vector(range) || !is.numeric(range)) {
    stop("`tipping_range` must be a numeric vector.")
  }
  if (any(range < 0)) {
    stop("All values in `tipping_range` must be greater than 0, i.e., deflation cannot be greater than 100%.")
  }
  return(range)
}

library(testthat)
library(survival)
library(dplyr)

# Cox model for testing
cox_fit <- coxph(Surv(AVAL, EVENT) ~ TRT01P, ties = "exact", data = codebreak200)

test_that("tipping_point_model_based returns correct structure for model-based imputations", {
  res <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    imputation_model = "weibull",
    J = 2,
    tipping_range = seq(0.1, 1, by = 0.1),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  expect_s3_class(res, "tipse")
  expect_true(all(c(
    "original_data", "original_HR", "reason_to_impute", "arm_to_impute",
    "method_to_impute", "imputation_results", "imputation_data"
  ) %in% names(res)))

  # Check imputation_results structure
  expect_true(all(c("HR", "HR_upperCI", "HR_lowerCI", "parameter_docetaxel", "tipping_point") %in% names(res$imputation_results)))

  # imputation_data should be a named list
  expect_true(is.list(res$imputation_data))
  expect_equal(length(res$imputation_data), length(seq(0.1, 1, by = 0.1)))
})

test_that("tipping_point_model_based throws error for invalid cox_fit", {
  expect_error(
    tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "docetaxel",
      imputation_model = "weibull",
      J = 2,
      tipping_range = seq(0.1, 1, by = 0.1),
      cox_fit = "not_a_coxph"
    ),
    "Argument 'cox_fit' must be a valid cox model object"
  )
})

test_that("tipping_point_model_based throws error for invalid impute arm", {
  expect_error(
    tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "invalid_arm",
      imputation_model = "weibull",
      J = 2,
      tipping_range = seq(0.1, 1, by = 0.1),
      cox_fit = cox_fit
    ),
    "Argument 'impute' must be one or both of the arms"
  )
})

test_that("tipping_point_model_based throws error when reason is empty", {
  expect_error(
    tipping_point_model_based(
      dat = codebreak200,
      reason = character(0),
      impute = "sotorasib",
      imputation_model = "weibull",
      J = 2,
      tipping_range = seq(0.1, 1, by = 0.1),
      cox_fit = cox_fit
    ),
    "Argument 'reason' must specify at least one censoring reason"
  )
})

test_that("tipping_point_model_based validates distribution argument", {
  expect_error(
    tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "sotorasib",
      imputation_model = "invalid_dist",
      J = 2,
      tipping_range = seq(0.1, 1, by = 0.1),
      cox_fit = cox_fit
    ),
    "'arg' should be one of \"weibull\", \"exponential\""
  )
})

test_that("tipping_point_model_based sets tipping_point flag correctly", {
  res <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    imputation_model = "weibull",
    J = 2,
    tipping_range = seq(0.1, 1, by = 0.1),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  expect_true("tipping_point" %in% names(res$imputation_results))
  expect_true(any(res$imputation_results$tipping_point %in% c(TRUE, FALSE)))
})

test_that("tipping_point_model_based respects seed for reproducibility", {
  res1 <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "sotorasib",
    imputation_model = "weibull",
    J = 2,
    tipping_range = seq(0.1, 1, by = 0.1),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  res2 <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "sotorasib",
    imputation_model = "weibull",
    J = 2,
    tipping_range = seq(0.1, 1, by = 0.1),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  expect_equal(res1$imputation_results, res2$imputation_results)
})

test_that("tipping_point_model_based detects method correctly based on tipping_range", {
  res_deflation <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "docetaxel",
    imputation_model = "weibull",
    J = 2,
    tipping_range = seq(0.05, 1, by = 0.05),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )
  expect_equal(res_deflation$method_to_impute, "hazard multiplication")

  res_inflation <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = "sotorasib",
    imputation_model = "weibull",
    J = 2,
    tipping_range = c(1.1, 1.2, 1.5),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )
  expect_equal(res_inflation$method_to_impute, "hazard multiplication")
})

test_that("tipping_point_model_based two-arm: returns correct structure", {
  res <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"),
    imputation_model = "weibull",
    J = 2,
    tipping_range = c(0.7, 0.9),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  expect_s3_class(res, "tipse")
  expect_equal(res$arm_to_impute, c("docetaxel", "sotorasib"))
  expect_equal(res$method_to_impute, "hazard multiplication")

  # Two-arm results should have arm-named parameter columns, not generic ones
  expect_true("parameter_docetaxel" %in% names(res$imputation_results))
  expect_true("parameter_sotorasib" %in% names(res$imputation_results))
  expect_false("parameter" %in% names(res$imputation_results))
  expect_false("parameter_1" %in% names(res$imputation_results))

  # Grid: 2 x 2 = 4 rows
  expect_equal(nrow(res$imputation_results), 4L)
  expect_equal(length(res$imputation_data), 4L)
})

test_that("tipping_point_model_based two-arm: per-arm list tipping_range produces Cartesian grid", {
  res <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"),
    imputation_model = "weibull",
    J = 2,
    tipping_range = list(c(0.6, 0.7, 0.8), c(0.8, 0.9)),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  # 3 x 2 = 6 rows
  expect_equal(nrow(res$imputation_results), 6L)
  expect_equal(length(unique(res$imputation_results$parameter_docetaxel)), 3L)
  expect_equal(length(unique(res$imputation_results$parameter_sotorasib)), 2L)
})

test_that("tipping_point_model_based two-arm: mixed direction (deflation + inflation) runs without error", {
  res <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"),
    imputation_model = "weibull",
    J = 2,
    tipping_range = list(c(0.7, 0.9), c(1.1, 1.3)),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  expect_s3_class(res, "tipse")
  expect_equal(nrow(res$imputation_results), 4L)
  expect_true(all(res$imputation_results$parameter_docetaxel <= 1))
  expect_true(all(res$imputation_results$parameter_sotorasib >= 1))
})

test_that("tipping_point_model_based two-arm: km_data_list names use pipe-separated arm parameters", {
  res <- tipping_point_model_based(
    dat = codebreak200,
    reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"),
    imputation_model = "weibull",
    J = 2,
    tipping_range = c(0.8, 0.9),
    cox_fit = cox_fit,
    verbose = FALSE,
    seed = 123
  )

  expect_true(all(grepl("|", names(res$imputation_data), fixed = TRUE)))
})

test_that("tipping_point_model_based two-arm: errors on duplicate arms", {
  expect_error(
    tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = c("docetaxel", "docetaxel"),
      imputation_model = "weibull",
      J = 2,
      tipping_range = c(0.7, 0.9),
      cox_fit = cox_fit
    ),
    "must not contain the same arm twice"
  )
})

test_that("tipping_point_model_based two-arm: errors when list tipping_range has wrong length", {
  expect_error(
    tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = c("docetaxel", "sotorasib"),
      imputation_model = "weibull",
      J = 2,
      tipping_range = list(c(0.7, 0.9)),
      cox_fit = cox_fit
    ),
    "exactly two elements"
  )
})

test_that("tipping_point_model_based two-arm: list tipping_range rejected for single arm", {
  expect_error(
    tipping_point_model_based(
      dat = codebreak200,
      reason = "Early dropout",
      impute = "docetaxel",
      imputation_model = "weibull",
      J = 2,
      tipping_range = list(c(0.7, 0.9), c(0.8, 1.0)),
      cox_fit = cox_fit
    ),
    "only allowed when two arms"
  )
})

test_that("tipping_point_model_based two-arm: reproducible with seed", {
  args <- list(
    dat = codebreak200, reason = "Early dropout",
    impute = c("docetaxel", "sotorasib"), imputation_model = "weibull",
    J = 2, tipping_range = c(0.7, 0.9), cox_fit = cox_fit,
    verbose = FALSE, seed = 42
  )
  res1 <- do.call(tipping_point_model_based, args)
  res2 <- do.call(tipping_point_model_based, args)
  expect_equal(res1$imputation_results, res2$imputation_results)
})

---
title: "Tipping point analysis examples"
date: "Last compiled on `r format(Sys.time(), '%d %B, %Y')`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tipping point analysis examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE, message = FALSE,
  fig.width = 6, fig.height = 4,
  comment = "#>"
)
```

```{r setup, include = FALSE}
library(tipse)
library(survival)
library(dplyr)
library(ggplot2)
```

# Introduction

This vignette demonstrates how to use the `tipse` package to perform
**tipping point analyses** as a sensitivity analysis for time-to-event
(survival) data. Tipping point analysis is an approach to evaluate how
sensitive treatment effect estimates are to different censoring
scenarios. It aims to evaluate the robustness of trial conclusions by
varying certain data and/or model aspects while imputing missing data.
This is particularly useful in clinical trials where dropout or loss to
follow-up may bias efficacy results.

The `tipse` package provides a flexible framework to impute censored
observations under different approaches and identify the tipping point
where the upper confidence limit (CL) of the hazard ratio crosses 1. It also offers
visualizations and plausibility assessment facilitate the interpretation
of tipping points.

Two approaches are implemented:

1.  **Model-Free Tipping Point Analysis**\
    Implemented via `tipping_point_model_free` and does not assume a
    parametric survival model. Uses either resampling of observed event
    times (`percentile sampling`) or landmark imputation of a fixed
    number of censored patients (`landmark sampling`).

2.  **Model-Based Tipping Point Analysis**\
    Implemented via `tipping_point_model_based` and uses parametric
    survival models (e.g., Weibull) to adjust the imputation of censored
    observations, allowing hazard inflation or deflation across a range
    of parameters.

## Example Dataset

we extracted data from the published Kaplan-Meier (KM) curve in de
Langen et al. [2023] using a KM digitizer. This re-constructed dataset
contains:

-   `TRT01P`: treatment arm assignment
-   `AVAL`: observed event time
-   `EVENT`: event indicator (1=event, 0=censored)
-   `CNSRRS`: censoring reason
-   `MAXAVAL`: maximum potential survival time (duration between randomization to data cut-off)

```{r data}
knitr::kable(codebreak200[1:5, ])
```

There are 21 (12.1 %) patients in the docetaxel control arm and 6 (3.5
%) patients in the sotorasib treatment arm who dropped out early on,
leading to potential information censoring. This raises concerns about
the validity of trial outcomes and robustness of treatment effects.

```{r data-censor}
codebreak200 %>% count(TRT01P, CNSRRS)
```

## Fit Initial Cox Model

We first fit a Cox proportional hazards model, which should have the
same model specification (including covariates and stratification factors) as the
original analysis. This model will be used for pooling hazard ratios
after multiple imputations. In this example, we do not include any covariates besides the treatment arm itself.

```{r cox-fit}
cox1 <- coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
```

# Model-Free Tipping Point Analysis

There are two methods available in `tipping_point_model_free`.

For **`method = "percentile sampling"`**, the key idea is to impute all
censored observations belonging to the arm specified in `impute` by
randomly drawing event times from either the best or worst percentiles
of the observed data, where observations are ranked according to their
event and censoring times.

For **`method = "landmark sampling"`**, the idea is to modify the
censored times directly: - when imputing the **control arm**, the
censoring times of a subset or all relevant censored observations are
extended to the maximum follow-up time; - when imputing the **treatment
arm**, the censoring time is instead treated as the event time.

| **Steps**                                        | **percentile Sampling**                                                                                                                                                                                                                                                                                                              | **landmark Sampling**                                                                                                                                                                                                             |
|----------------|---------------------------|------------------------------|
| **1. Identify censored patients for imputation** | Select all censored observations in the arm specified by the `impute` argument that correspond to a particular censoring reason (e.g., early dropout). Assume n patients are identified.                                                                                                                                         | Same as left                                                                                                                                                                                                                           |
| **2. Define tipping parameter range**            | Specifies the percentiles of the observed data from which event times will be sampled to impute censored patients.<br> \* For **treatment arm**, use the worst percentiles (shortest survival times) from the observed data of both arms.<br> \* For **control arm**, use the best percentiles (longest survival times). | \* For **treatment arm**, define the number of patients that will be assumed to have an event at their time of censoring.<br> \* For **control arm**, define the number of patients that will be assumed to be event-free at DCO. |
| **3. Sampling / Imputation**                     | For each value in `tipping_range` impute patients using event times corresponding up to that percentile.                                                                                                                                                                                                                         | For each value in `tipping_range`, randomly select `x` patients from the `n` censored patients to impute as event-free at DCO.<br>The remaining `n - x` patients are not imputed.                                                      |
| **4. Multiple imputations**                      | Repeat step 3 `J` times.                                                                                                                                                                                                                                                                                                         | Same as left                                                                                                                                                                                                                           |
| **5. Model fitting and pooling**                 | For each imputed dataset, fit the Cox proportional hazards model specified via the `cox_fit` argument).<br>Pool the results across the `J` replicates using **Rubin's rules**, producing a combined hazard ratio (HR) and confidence interval for that tipping parameter.                                                        | Same as left                                                                                                                                                                                                                           |
| **6. Iterate across tipping points**             | Repeat steps 2--5 across all values in the `tipping_range`.<br>This generates a series of pooled HRs and confidence intervals showing how the treatment effect changes as increasingly conservative imputation.                                                                                                                  | Same as left                                                                                                                                                                                                                           |
| **7. Identify the tipping point**                | **tipping point** is the parameter at which the upper bound first crosses 1, indicating the loss of the apparent treatment benefit.                                                                                                                                                                                              | Same as left                                                                                                                                                                                                                           |
| **8. Visualize and assess**                      | Plot pooled Kaplan-Meier curves to visually inspect how much shift the imputation created compared to the original data, and how HR changes over different tipping parameters using `plot`.<br>Assess the plausibility of such tipping point occurs using `assess_plausibility`.                                                 | Same as left                                                                                                                                                                                                                           |

### Step-by-step walk through of the analyses using model-free imputation

```{r tipping-point-percentile, cache=TRUE}
tp_percentile <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  J = 100,
  tipping_range = seq(5, 95, by = 5),
  cox_fit = cox1,
  method = "percentile sampling",
  seed = 12345
)
```

```{r tipping-point-landmark, cache=TRUE}
tp_landmark <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  J = 100,
  tipping_range = 1:20,
  cox_fit = cox1,
  method = "landmark sampling",
  seed = 12345
)
```

## Visualize Pooled Kaplan--Meier Curves

```{r plot-percentile-km, cache=TRUE}
plot(tp_percentile, type = "Kaplan-Meier")
```

```{r plot-landmark-km, cache=TRUE}
plot(tp_landmark, type = "Kaplan-Meier")
```

While there is no objective measure to assess the robustness of the
result, the average KM curves give a visual representation of how
optimistic the assumptions of the tipping point approaches are on the
control arm.

**Interpretation of the curves:**

-   **Red curve**: tipping point parameter where the upper CL of the
    hazard ratio crosses 1 (i.e., the point at which treatment effect is
    no longer statistically significant).\
-   **Orange dashed curve**: original (unimputed) Kaplan--Meier survival
    for the imputed arm.\
-   **Gradient blue curves**: pooled Kaplan--Meier estimates across all
    parameter values.

## Visualize Tipping Point Graph

```{r plot-percentile-tp, cache=TRUE}
plot(tp_percentile, type = "Tipping Point")
```

```{r plot-landmark-tp, cache=TRUE}
plot(tp_landmark, type = "Tipping Point")
```

## Summarize tipping point via Analysis Results Dataset (ARD)

```{r summary-model-free, cache=TRUE}
summary(tp_percentile)
summary(tp_landmark)
```

The model-free approaches suggest that the results tip if at least `r summary(tp_landmark)$TIPPT` out
of 21 (`r round(summary(tp_landmark)$TIPPT/21*100,0)` %) of early dropout patients in the control arm are considered
as event-free at the data cut-off or if we impute all of the early
dropout in the arm from a sample of `r summary(tp_percentile)$TIPPT` % best event times (both arms
combined).

## Assess plausibility

```{r assess-model-bree}
assess_plausibility(tp_percentile)
assess_plausibility(tp_landmark)
```


# Model-Based Tipping Point Analysis

Model-based tipping point analysis uses parametric survival models to
generate event times for censored patients. This approach allows
systematic hazard inflation or deflation, known as the delta-adjustment
method [Lipkovich et al.
(2016)](https://onlinelibrary.wiley.com/doi/full/10.1002/pst.1738).

Compared to the steps described in the table above for model-free tipping point analysis, only step 2 and 3 differs:

- **Step 2 Define tipping parameter range**: 

For treatment arm, define the hazard inflation $\delta>1$ for the patients' follow-up time $t>c$ after censoring. For control arm, define the hazard deflation $\delta<1$ for the patients' follow-up time $t>c$ after censoring. 
$$\hat{h}(t|t > c) = \delta\hat{h}(t)$$

- **Step 3 Sampling / Imputation**: 

*Fitting parametric model*

For a Weilbull model with shape parameter $p$ and scale parameter $\lambda$, the cumulative hazard, survival and cumulative distribution functions are
$$h(t) = \lambda p (\lambda t)^{(p-1)}, H(t) = (\lambda t)^p, S(t) = \exp(-(\lambda t)^p), F(t)=1-\exp(-(\lambda t)^p)$$
A Weibull model is fitted to the treatment arm patients to obtain empirical estimates of $p$ and $\lambda$, the goal is to impute event times for a subset of censored patients based on the Weibull model using inverse probability sampling.

*Impute event time under a Weibull model*

Assuming a patient is censored at time $c$ and we wish to impute the event time for this patient. We evaluate the cumulative distribution function at the survival time $F(c)$. Using inverse probability sampling, we generate $u_d$ from a uniform distribution $U(F(c), 1)$ and solve $d$ from the equation
$$u_d = F(d) = 1-\exp(-(\lambda d)^p)$$
The solution $d$ is then a imputed event time based on the cumulative distribution function $F(d)$ from a Weibull model. For each imputation, the Weibull model parameters $p$ and $\lambda$ are re-sampled from a multivariate normal distribution with their maximum likelihood estimates and covariate matrix. 

*Impute event time with inflated hazard*

In order to inflate the hazard after time $c$, we assume a two-piece Weibull model where the original hazard is observed until time $c$ as the first piece, and for $t\geq c$ the hazard function $h(t)$ is inflated via $\alpha$ as the second piece to become $h_i(t) = \alpha h(t)$. This inflation is equivalent to inflating on the cumulative hazard scale via
$$H_i(t) = H(c) + \alpha (H(t) - H(c))$$
The imputed event time $d$ can be found by solving the cumulative distribution function $F(d) = 1-\exp(-H_i(d))$. 

```{r tipping-point-hazard-deflation, cache=TRUE}
tp_model_based <- tipping_point_model_based(
  dat = codebreak200, reason = "Early dropout",
  impute = "docetaxel",
  imputation_model = "weibull",
  J = 100,
  tipping_range = seq(0.1, 1, by = 0.1),
  cox_fit = cox1,
  seed = 12345
)
```

## Visualize Pooled Kaplan--Meier Curves

```{r plot-model-based-KM, cache=TRUE}
plot(tp_model_based, type = "Kaplan-Meier")
```

## Visualize Tipping Point Graph

```{r plot-model-based-hazard-inflation, cache=TRUE}
plot(tp_model_based, type = "Tipping Point") + scale_x_reverse()
```

The model-based approach tips the results with a hazard deflation of at
least `r summary(tp_model_based)$TIPPT`% for imputing event times for early dropout patients in the
control arm.

## Summary via Analysis Results Dataset (ARD)

```{r summary-model-based, cache=TRUE}
summary(tp_model_based)
```

## Assess plausibility

```{r assess-model-based, cache=TRUE}
assess_plausibility(tp_model_based)
```

To assess the clinical plausibility on the tipping point from
model-based approach using the trial data, `r summary(tp_model_based)$TIPPT`% hazard deflation
translates to a HR of `r 1-summary(tp_model_based)$TIPPT/100` between early dropouts in control arm and
control arm patients who did not drop out. This translated to HR = `r 1-summary(tp_model_based)$TIPPT/100`
/ `r round(summary(cox1)$coefficients[, "exp(coef)"],2)` = `r round((1-summary(tp_model_based)$TIPPT/100)/summary(cox1)$coefficients[, "exp(coef)"], 2)` between the early dropout patients in the control arm and
Sotorasib arm, which seems unlikely given the limited treatment options
these patients have.

# Jump-to-Reference (J2R) as a Special Case of Model-based Imputation

The Jump-to-Reference (J2R) method assumes that, upon censoring, subjects in the treatment arm behave as if they had switched to the reference (usually control) arm. In a time-to-event framework, this corresponds to replacing the post-censoring hazard for treatment-arm dropouts with the hazard estimated from the reference arm.

Importantly, J2R is a **special case of hazard inflation**, where the inflation factor is chosen to exactly match the reference arm hazard. Assuming a Cox model, the hazard inflation needed to convert treatment-arm hazard into reference-arm hazard is:

\[
\text{hazard inflation} = \frac{1}{\widehat{HR}}
\]

Thus, J2R can be implemented in the tipping-point framework by setting the tipping-range to \(1/HR\), meaning the hazard is inflated until treatment arm dropouts “jump” to the reference hazard.

Below we illustrate this using a Weibull imputation model and the treatment-arm dropout reason “Lost to follow-up”.

## Inspect the data

```{r}
knitr::kable(extenet[seq(1, 2000, by = 400), ])
```

## Fit the original Cox model

```{r}
cox2 <- coxph(Surv(AVAL, EVENT) ~ TRT01P, data = extenet)
summary_cox2 <- summary(cox2)

## Extract HR for treatment vs reference
orig_HR <- summary_cox2$coefficients[, "exp(coef)"]
orig_HR
```

J2R implemented as hazard inflation = 1 / HR

```{r}
j2r_inflation <- 1 / orig_HR
j2r_inflation
```

## Run model-based imputation

We see the parameter `tipping_range` to the J2R inflation to produce the J2R scenario,

```{r, cache = TRUE}
j2r <- tipping_point_model_based(
  dat = extenet,
  reason = "Lost to follow-up",
  impute = "neratinib", # treatment arm with dropouts
  imputation_model = "weibull",
  J = 100,
  tipping_range = j2r_inflation, # J2R through hazard inflation
  cox_fit = cox2,
  seed = 12345
)
```

## J2R Results

The output below shows the HR estimated in the J2R scenario. The hazard ratio changed from `r round(orig_HR, 3)` in the original analysis to `r round(j2r$imputation_results$HR, 3)` in the jump-to-reference imputation.


```{r}
j2r$imputation_results
```


# References
-   I. Lipkovich, B. Ratitch, and M. O'Kelly. Sensitivity to
    censored-at-random assumption in the analysis of time- to-event
    endpoints. Pharmaceutical Statistics, 15(3):216--229, 2016.
    <https://doi.org/10.1002/pst.1738>.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_model.R
\name{fit_model}
\alias{fit_model}
\title{Fit parametric model for selected subjects}
\usage{
fit_model(dat, reason, impute, imputation_model = c("weibull", "exponential"))
}
\arguments{
\item{dat}{data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)}

\item{reason}{a string specifying the censoring reasons which require imputation. It must be one of the reasons from column CNSRRS.}

\item{impute}{a string specifying the treatment arm(s) which require imputation. It must be one of the arms from column TRT01P.}

\item{imputation_model}{a string specifying the parametric distribution used for imputation, can be "Weibull" or "exponential".}
}
\value{
data.frame with flags and fitted model parameters to be used for imputation
}
\description{
Fit parametric model for selected subjects
}
\details{
The data.frame contains original columns, plus the following columns appended:
\tabular{ll}{
AVAL4  \tab Placeholder column to keep imputed survival times \cr
EVENT4 \tab Placeholder column to keep imputed events \cr
impute \tab Flag indicating whether the subject was selected for imputation \cr
a      \tab Shape parameter, equal to 1 if exponential \cr
b      \tab Scale parameter \cr
cdf    \tab Cumulative distribution function \cr
...    \tab Some temporary columns
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess_plausibility.R
\name{assess_plausibility}
\alias{assess_plausibility}
\title{Assess Clinical Plausibility of Imputation Results}
\usage{
assess_plausibility(tipse, verbose = TRUE)
}
\arguments{
\item{tipse}{A \code{tipse} object returned by one of \link{tipping_point_model_free} or \link{tipping_point_model_based}.}

\item{verbose}{Logical. If \code{TRUE}, prints assessment details.}
}
\value{
A character string summarizing the key information to facilitate clinical plausibility assessment based on
the imputation scenario.
}
\description{
This function facilitates the evaluation of clinical plausibility at the tipping point. It provides a text summary comparing event rates,
follow-up duration, or hazard ratios between treatment arms depending on the
imputation method and arm specified. NOTE: this function only supports imputation in one arm.
}
\examples{
cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
result <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  cox_fit = cox1,
  method = "percentile sampling"
)

assess_plausibility(result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tipping_point_model_free.R
\name{tipping_point_model_free}
\alias{tipping_point_model_free}
\title{Tipping Point Analysis (Model-Free)}
\usage{
tipping_point_model_free(
  dat,
  reason,
  impute,
  J = 10,
  tipping_range = seq(5, 95, by = 5),
  cox_fit = NULL,
  verbose = FALSE,
  method = c("percentile sampling", "landmark sampling"),
  seed = NULL
)
}
\arguments{
\item{dat}{data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)}

\item{reason}{Vector specifying censoring reasons to be imputed.}

\item{impute}{a character vector specifying the arm(s) to impute. Can be one arm or both arms (a length-2 vector). Each value must be one of the arms from variable TRT01P. When both arms are supplied, imputation is applied independently for each arm.}

\item{J}{numeric indicating number of imputations.}

\item{tipping_range}{Numeric vector or when \code{length(impute) == 2} optionally a named or unnamed list of two numeric vectors (one per arm). When a list is supplied, all combinations of the two vectors are evaluated. Percentiles to use when \code{method = "percentile sampling"}. Number of patients to impute when \code{method = "landmark sampling"}.}

\item{cox_fit}{A Cox model that will be used to calculate HRs on imputed datasets.
In case of inclusion of stratification factors or covariates, conditional HR will be used.}

\item{verbose}{Logical. If \code{TRUE}, prints progress and analysis details.}

\item{method}{Character. Either \code{"percentile sampling"} or \code{"landmark sampling"}.}

\item{seed}{Integer, default as NULL. Random seed for reproducibility.}
}
\value{
A \code{tipse} object containing:
\describe{
\item{original data}{Input argument from 'data'.}
\item{imputation_results}{A data frame of combined pooled model results across tipping points}
\item{original_HR}{The original hazard ratio.}
\item{reason_to_impute}{Input argument from 'reason'.}
\item{arm_to_impute}{Input argument from 'impute'.}
\item{method_to_impute}{Input argument from 'method'.}
\item{imputation_data}{A list of imputed datasets for each tipping point value.}
\item{seed}{Random seed.}
}
}
\description{
Performs a model-free tipping point analysis on time-to-event data by
repeatedly imputing censored observations under varying assumptions. The model-free
framework assumes that censored patients share similar survival
behavior with those from whom they are sampled, without fitting any parametric
survival model.
}
\details{
The \strong{model-free tipping point analysis} provides a reproducible and intuitive
framework for exploring the robustness of treatment effects in time-to-event
(survival) endpoints when censoring may differ between study arms.

Two sampling modes are supported:
\itemize{
\item \code{method = "percentile sampling"} - performs re-sampling of event
times from the best or worst percentile of observed patients ranked by their event or censoring time.
The \code{tipping_range} specifies the percentiles of the observed data from which
event times will be sampled to impute censored patients.
For the treatment arm, use the worst percentiles (shortest survival times) from the
observed data of both arms. For the control arm, use the best percentiles (longest survival times).
\item \code{method = "landmark sampling"} - imputes a fixed number of censored
patients deterministically. The \code{tipping_range} specifies the number of patients to be imputed.
For the treatment arm, it defines the number of patients that will be assumed to
have an event at their time of censoring. For the control arm, it defines the
number of patients that will be assumed to be event-free at data cut-off, their maximum potential follow-up time.
}

This function iteratively applies the percentile- or landmark-sampling
imputation procedure across a range of
tipping point parameters \code{tipping_range}. For each parameter value:
\enumerate{
\item Multiple imputed datasets are generated (\code{J} replicates), where censored
observations in the selected arm are replaced by sampled or reassigned
event times according to the imputation method.
\item A Cox proportional hazards model is fitted to each imputed dataset.
\item Model estimates are pooled using \strong{Rubin’s rules} to obtain a combined
hazard ratio and confidence interval for that tipping point parameter.
}

The process yields a series of results showing how the treatment effect changes
as increasingly conservative or optimistic assumptions are made about censored
observations. The \emph{tipping point} is defined as the smallest value of the
sensitivity parameter (percentile or number of imputed patients) for which the upper
bound of the hazard ratio confidence interval crosses 1 - i.e., where the
apparent treatment benefit is lost.
}
\examples{
cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
result <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  J = 10,
  tipping_range = seq(5, 95, by = 5),
  cox_fit = cox1,
  method = "percentile sampling"
)

result2 <- tipping_point_model_free(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  J = 10,
  tipping_range = seq(1, 21, by = 2),
  cox_fit = cox1,
  method = "landmark sampling"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute_percentile.R
\name{impute_percentile}
\alias{impute_percentile}
\title{Model-free imputation via percentile sampling}
\usage{
impute_percentile(dat, reason, impute, percentile, J, seed = NULL)
}
\arguments{
\item{dat}{data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)}

\item{reason}{a string specifying the censoring reasons which require imputation. It must be one of the reasons from variable CNSRRS.}

\item{impute}{a string specifying the treatment arm(s) which require imputation. It must be one of the arms from variable TRT01P, the first level of TRT01P is considered as the control arm.}

\item{percentile}{numeric between 1 and 100, indicating the best (or worst) percentile of subjects to sample from.}

\item{J}{numeric indicating number of imputations.}

\item{seed}{Integer. Random seed for reproducibility.}
}
\value{
a list of data.frame from each imputation with imputed AVAL and EVENT, where original variables are kept as AVALo and EVENTo.
}
\description{
randomly sample from the percentile of best or worst patients (ordered by their observed times regardless of event or censoring) who do not require imputation.
}
\details{
We define two sets of subjects to sample from depending on the \code{impute} argument:
\enumerate{
\item \strong{Worst percentile of observations from treatment arm}
\eqn{ \forall i \in N \mid \min\{T_i, C_i\} \leq F_{\min\{T_i, C_i\}}^{-1}(\kappa) }.
This set includes all indices \eqn{ i } where the minimum of \eqn{ T_i } (event time) and \eqn{ C_i } (censoring time) is \strong{less than or equal to} the \eqn{\kappa}-th percentile of its distribution.
\item \strong{Best percentile of observations control arm}
\eqn{ \forall i \in N \mid \min\{T_i, C_i\} \geq F_{\min\{T_i, C_i\}}^{-1}(\kappa) }.
This set includes all indices \eqn{ i } where the minimum of \eqn{ T_i } and \eqn{ C_i } is \strong{greater than or equal to} the \eqn{\kappa}-th percentile of its distribution.
}

where \eqn{ F(\cdot) } denotes the cumulative distribution function (CDF) of the observed times and \eqn{F^{-1}(\kappa) } is the inverse CDF (quantile function) at percentile \eqn{\kappa}.
}
\examples{
impute_percentile(
  dat        = codebreak200,
  reason     = "Early dropout",
  impute     = "docetaxel",
  percentile = 30,
  J          = 5,
  seed       = 1
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_tp_2d.R
\name{plot_tp_2d}
\alias{plot_tp_2d}
\title{Heatmap of Two-Dimensional Tipping Point Analysis}
\usage{
plot_tp_2d(
  tipse,
  title = "Two-Dimensional Tipping Point Analysis",
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  cross = TRUE,
  line = TRUE
)
}
\arguments{
\item{tipse}{A \code{tipse} object returned by \link{tipping_point_model_based}
or \link{tipping_point_model_free} with \code{impute} of length 2.}

\item{title}{Optional character string specifying the plot title. Set to \code{NULL}
to remove the title.}

\item{subtitle}{Optional character string specifying the plot subtitle. If \code{NULL},
a default subtitle including the method and tipping point is used.}

\item{xlab}{Optional character string specifying the x-axis label. If \code{NULL},
a method-specific default is used.}

\item{ylab}{Optional character string specifying the y-axis label. Defaults to
\code{"Hazard Ratio"}.}

\item{cross}{Optional logical to specifying if tipping points will be marked with a cross.}

\item{line}{Optional logical to specifying if tipping line will be marked with a contour.}
}
\value{
A \code{ggplot2} object.
}
\description{
Visualises the upper confidence limit of the hazard ratio across a Cartesian
grid of two imputation parameters (one per arm). Cells are shaded by
\code{HR_upperCL}, with the tipping line (where \code{HR_upperCI} crosses 1)
highlighted by a dashed contour line and flagged cells marked with a cross.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_assess_plausibility.R
\name{print.plausibility_assessment}
\alias{print.plausibility_assessment}
\title{Print method for plausibility assessment}
\usage{
\method{print}{plausibility_assessment}(x, ...)
}
\arguments{
\item{x}{An object of class "plausibility_assessment"}

\item{...}{Further arguments passed to or from other methods.}
}
\description{
Print method for plausibility assessment
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codebreak200.R
\docType{data}
\name{codebreak200}
\alias{codebreak200}
\title{Patient level data from dummy trial}
\format{
A data frame with 345 rows and 5 columns:
\describe{
\item{SUBJID}{Dummy patient ID}
\item{TRT01P}{Treatment arm (Sotorasib or Docetaxel)}
\item{AVAL}{PFS time in days}
\item{EVENT}{Indicator for PFS event}
\item{CNSRRS}{Censoring reason (Early dropout or Other)}
\item{MAXAVAL}{Maximum potential survival time, duration between randomization to data cut-off}
}
}
\source{
De Langen, A.J., Johnson, M.L., Mazieres, J., Dingemans, A.M.C., Mountzios, G., Pless, M., Wolf, J., Schuler, M., Lena, H., Skoulidis, F. and Yoneshima, Y., 2023. Sotorasib versus docetaxel for previously treated non-small-cell lung cancer with KRASG12C mutation: a randomised, open-label, phase 3 trial. The Lancet, 401(10378), pp.733-746.
}
\usage{
codebreak200
}
\description{
Based on re-constructed Kaplan-Meier plot from CodeBreak 200 trial (de Langen et al., 2023)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_tipse.R
\name{summary.tipse}
\alias{summary.tipse}
\title{Summarize Tipping Point Results (ARD Format)}
\usage{
\method{summary}{tipse}(object, ...)
}
\arguments{
\item{object}{A \code{tipse} object returned by \link{tipping_point_model_free} or
\link{tipping_point_model_based}.}

\item{...}{Additional arguments not used.}
}
\value{
A data frame summarizing:
\itemize{
\item \code{HR} - hazard ratio at that tipping point
\item \code{CONFINT} - 95\% CI at tipping point
\item \code{METHOD} - sampling type used
\item \code{ARMIMP} - arm imputed
\item \code{TIPPT} - parameter where upper CL first crosses 1
\item \code{TIPUNIT} - parameter meaning
\item \code{DESC} - textual interpretation
}
}
\description{
Creates a concise, analysis-results dataset (ARD) from a tipping point
analysis. Identifies the tipping point parameter where the upper CL of the hazard ratio crosses 1
and summarizes key metrics.
}
\examples{
\donttest{
# Hazard deflation in the control arm
cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
result1 <- tipping_point_model_based(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  imputation_model = "weibull",
  J = 10,
  tipping_range = seq(0.1, 1, by = 0.05),
  cox_fit = cox1
)

summary(result1)

# Imputation in both arms
result2 <- tipping_point_model_based(
  dat = codebreak200,
  reason = "Early dropout",
  impute = c("docetaxel", "sotorasib"),
  imputation_model = "weibull",
  J = 10,
  tipping_range = list(seq(0.1, 1, by = 0.2), seq(0.5, 1.5, by = 0.2)),
  cox_fit = cox1,
  verbose = TRUE,
  seed = 12345
)

summary(result2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute_landmark.R
\name{impute_landmark}
\alias{impute_landmark}
\title{Model-free imputation via landmark sampling}
\usage{
impute_landmark(dat, reason, impute, npts, J, seed)
}
\arguments{
\item{dat}{data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)}

\item{reason}{a string specifying the censoring reasons which require imputation. It must be one of the reasons from variable CNSRRS.}

\item{impute}{a string specifying the treatment arm(s) which require imputation. It must be one of the arms from variable TRT01P, the first level of TRT01P is considered as the control arm.}

\item{npts}{number of patients to be imputed}

\item{J}{numeric indicating number of imputations.}

\item{seed}{Integer. Random seed for reproducibility.}
}
\value{
a list of data.frame from each imputation with imputed AVAL and EVENT, where original variables are kept as AVAL and EVENT.
}
\description{
patients will be assigned deterministically an event time at the time of censoring  or extend the censoring time to the potential maximum follow-up of each patient.
}
\details{
patients will be assigned deterministically an event time at the time of censoring  or extend the censoring time to the potential maximum follow-up of each patient.
}
\examples{
impute_landmark(
  dat    = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  npts   = 5,
  J      = 5,
  seed   = 1
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_km.R
\name{plot_km}
\alias{plot_km}
\title{Plot Pooled Kaplan–Meier Curves from Model-Free Tipping Point Analysis}
\usage{
plot_km(tipse)
}
\arguments{
\item{tipse}{An S3 object of class \code{"tipse"} returned from \code{tipping_point_model_free()} or \code{tipping_point_model_based()}.}
}
\value{
A \code{ggplot2} object displaying pooled Kaplan-Meier curves, with:
\itemize{
\item Colored lines - pooled KM curves for each tipping point parameter
\item Red line - tipping point where HR upper CL crosses 1
\item Orange dashed line - original (unimputed) KM curve
}
}
\description{
Visualizes averaged (pooled) Kaplan-Meier survival curves across multiple tipping
point parameters, highlighting the tipping point where the upper CL of the hazard ratio crosses 1.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tipse-package.R
\docType{package}
\name{tipse-package}
\alias{tipse}
\alias{tipse-package}
\title{tipse: Tipping Point Analysis for Survival Endpoints}
\description{
Implements tipping point sensitivity analysis for time-to-event endpoints under different missing data scenarios, as described in Oodally et al. (2025) \doi{10.48550/arXiv.2506.19988}. Supports both model-based and model-free imputation, multiple imputation workflows, plausibility assessment and visualizations. Enables robust assessment for regulatory and exploratory analyses.
}
\author{
\strong{Maintainer}: Ajmal Oodally \email{ajmal.oodally@novartis.com} (\href{https://orcid.org/0000-0001-8143-5011}{ORCID})

Authors:
\itemize{
  \item Craig Wang \email{craig.wang@novartis.com} (\href{https://orcid.org/0000-0003-1804-2463}{ORCID})
}

Other contributors:
\itemize{
  \item Zheng Li \email{zheng.li@novartis.com} (\href{https://orcid.org/0000-0002-7655-0290}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pool_results.R
\name{pool_results}
\alias{pool_results}
\title{Pooling results using Rubin's Rule}
\usage{
pool_results(dat, cox.fit, conf.level = 0.95)
}
\arguments{
\item{dat}{a list of data.frames from multiple imputation using one alpha or kappa parameter}

\item{cox.fit}{a coxph object which is used to compute HRs for each imputed datasets}

\item{conf.level}{confidence level for the returned confidence interval, default to be 0.95.}
}
\value{
a data.frame of pooled hazard ratio and confidence interval estimate using Rubin's Rule
}
\description{
Pooling results from multiple imputations using Rubin's Rule
}
\details{
The Rubin's rule is applied to the Cox PH model results across imputed datasets as:

\enumerate{
\item \emph{Compute pooled HR:}
\deqn{\bar{HR}_\lambda = \exp\Bigg(\frac{1}{M} \sum_{m=1}^{M} \log(HR_m)\Bigg)}

\item \emph{Compute pooled variance:}
\deqn{\bar{\sigma}_\lambda^2 = \frac{1}{M} \sum_{m=1}^{M} \sigma_m^2 +
  \frac{1 + \frac{1}{M}}{M-1} \sum_{m=1}^{M} \big(\log(HR_m) - \overline{\log(HR_\lambda)}\big)^2}

\item \emph{Compute CI:}
\deqn{\bar{HR}_\lambda \times \exp\big(\pm t_{\alpha/2} \sqrt{\bar{\sigma}_\lambda^2}\big)}
}
}
\examples{
cox_fit <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
imputed_list <- impute_percentile(
  dat        = codebreak200,
  reason     = "Early dropout",
  impute     = "docetaxel",
  percentile = 30,
  J          = 5,
  seed       = 1
)
pool_results(imputed_list, cox_fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_tp.R
\name{plot_tp}
\alias{plot_tp}
\title{Plot Tipping Point Analysis Results}
\usage{
plot_tp(
  tipse,
  title = "Tipping Point Analysis",
  subtitle = NULL,
  xlab = NULL,
  ylab = "Hazard Ratio"
)
}
\arguments{
\item{tipse}{An S3 object of class \code{"tipse"} returned from
\code{tipping_point_model_free()} or \code{tipping_point_model_based()}.}

\item{title}{Optional character string specifying the plot title. Set to \code{NULL}
to remove the title.}

\item{subtitle}{Optional character string specifying the plot subtitle. If \code{NULL},
a default subtitle including the method and tipping point is used.}

\item{xlab}{Optional character string specifying the x-axis label. If \code{NULL},
a method-specific default is used.}

\item{ylab}{Optional character string specifying the y-axis label. Defaults to
\code{"Hazard Ratio"}.}
}
\value{
A \code{ggplot2} object showing hazard ratios and confidence intervals across
tipping point parameters. The returned plot can be further modified using \code{+}.
}
\description{
Visualizes hazard ratios and corresponding confidence intervals across tipping point
parameters from tipping point analyses (one-dimensional). Supports both model-free and model-based
approaches, including percentile sampling, landmark sampling, and hazard-based methods.
The tipping point—defined as the parameter value where the upper confidence interval
crosses 1—is highlighted on the plot.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extenet.R
\docType{data}
\name{extenet}
\alias{extenet}
\title{Patient level data from dummy trial}
\format{
A data frame with 2840 rows and 5 columns:
\describe{
\item{SUBJID}{Dummy patient ID}
\item{TRT01P}{Treatment arm (Neratinib or placebo)}
\item{AVAL}{iDFS time in days}
\item{EVENT}{Indicator for iDFS event}
\item{CNSRRS}{Censoring reason (Lost to follow-up or Other)}
\item{MAXAVAL}{Maximum potential survival time, duration between randomization to data cut-off}
}
}
\source{
Martin, M., Holmes, F.A., Ejlertsen, B., Delaloge, S., Moy, B., Iwata, H., von Minckwitz, G., Chia, S.K., Mansi, J., Barrios, C.H. and Gnant, M., 2017. Neratinib after trastuzumab-based adjuvant therapy in HER2-positive breast cancer (ExteNET): 5-year analysis of a randomised, double-blind, placebo-controlled, phase 3 trial. The lancet oncology, 18(12), pp.1688-1700.
}
\usage{
extenet
}
\description{
Based on re-constructed Kaplan-Meier plot from ExteNET trial (Martin et al., 2017)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute_model_based.R
\name{impute_model}
\alias{impute_model}
\title{Model-based imputation from parametric distributions}
\usage{
impute_model(
  dat,
  reason,
  impute,
  imputation_model = c("weibull", "exponential"),
  alpha,
  J,
  seed = NULL
)
}
\arguments{
\item{dat}{data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)}

\item{reason}{a string specifying the censoring reasons which require imputation. It must be one of the reasons from variable CNSRRS.}

\item{impute}{a string specifying the treatment arm(s) which require imputation. It must be one of the arms from variable TRT01P, the first level of TRT01P is considered as the control arm.}

\item{imputation_model}{a string specifying the parametric distribution used for imputation, can be "Weibull" or "exponential".}

\item{alpha}{hazard inflation (if treatment arm is imputed) or deflation (if control arm is imputed) rate}

\item{J}{numeric indicating number of imputations.}

\item{seed}{Integer. Random seed for reproducibility.}
}
\value{
a list of data.frame from each imputation with imputed AVAL and EVENT, where original variables are kept as AVALo and EVENTo.
}
\description{
Impute data with Weibull or exponential distribution conditional on follow-up time
}
\details{
First fit model based on the data without dropout. And then impute the survival outcome based on exponential or Weibull distribution for those who dropped out.
}
\examples{
impute_model(
  dat               = codebreak200,
  reason            = "Early dropout",
  impute            = "docetaxel",
  imputation_model  = "weibull",
  alpha             = 0.7,
  J                 = 5,
  seed              = 1
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/average_km.R
\name{average_km}
\alias{average_km}
\title{Average Kaplan-Meier Curves Across Multiple Imputed Datasets}
\usage{
average_km(km_data, arm, conf_level = 0.95)
}
\arguments{
\item{km_data}{List of data frames, each containing one multiply imputed dataset for a tipping point.
Each data frame must contain columns \code{AVAL}, \code{EVENT}, \code{TRT01P}, \code{iter}, and a tipping point parameter depending on the method.}

\item{arm}{Character string specifying the treatment arm to pool (must match \code{TRT01P} levels).}

\item{conf_level}{Numeric. Confidence level for CIs (default = 0.95).}
}
\value{
A data frame with the following columns:
\describe{
\item{time}{Time points of the KM curve.}
\item{survival_comb}{Pooled survival probability at each time point.}
\item{survival_lcl_comb}{Lower 95\% confidence limit of pooled survival.}
\item{survival_ucl_comb}{Upper 95\% confidence limit of pooled survival.}
\item{stderr}{Standard error of the pooled log-log transformed estimate.}
}
}
\description{
Takes a list of multiply imputed datasets corresponding to a single tipping point
parameter and pools the Kaplan-Meier survival curves for a given treatment arm.
Uses log(-log) transformation and Rubin's rules for pooling across imputations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_tipse.R
\name{plot.tipse}
\alias{plot.tipse}
\title{Plot Pooled Kaplan–Meier Curves from Tipping Point Analysis}
\usage{
\method{plot}{tipse}(x, type = c("Kaplan-Meier", "Tipping Point"), ...)
}
\arguments{
\item{x}{An S3 object of class \code{"tipse"} returned from \link{tipping_point_model_free} or \link{tipping_point_model_based}.}

\item{type}{Type of plot, either "Kaplan-Meier" or "Tipping Point".}

\item{...}{Additional arguments to specify title, subtitle, xlab and ylab.}
}
\value{
A ggplot2 object displaying pooled Kaplan–Meier curves.
}
\description{
Visualizes averaged (pooled) Kaplan-Meier survival curves across multiple tipping
point parameters, highlighting the tipping point where the upper CL of the hazard ratio crosses 1.
}
\details{
\itemize{
\item If \code{type = Kaplan-Meier}, then the KM curves from multiply imputed datasets were pooled using Rubin’s rules
after complementary log-log transformation as described in Marshall et al. (2009).
it can be of interest to visually assess the scenario that tips the result and the shift it causes to the original KM curve,
although there is no objective measure to assess the robustness of the result.
\item If \verb{type = Tipping Point}, then the HR estimation across the range of tipping point parameters are plotted.
}
}
\examples{
\donttest{
cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
result <- tipping_point_model_based(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  imputation_model = "weibull",
  J = 10,
  tipping_range = seq(0.1, 1, by = 0.05),
  cox_fit = cox1
)

plot(result, type = "Kaplan-Meier")
plot(result, type = "Tipping Point")


# Imputation in both arms
result2 <- tipping_point_model_based(
  dat = codebreak200,
  reason = "Early dropout",
  impute = c("docetaxel", "sotorasib"),
  imputation_model = "weibull",
  J = 10,
  tipping_range = list(seq(0.4, 0.7, by = 0.1), seq(0.5, 1.5, by = 0.2)),
  cox_fit = cox1
)

plot(result2, type = "Tipping Point")
}

}
\references{
Marshall, A., Altman, D.G., Holder, R.L. et al.
Combining estimates of interest in prognostic modelling studies after multiple imputation: current practice and guidelines.
BMC Med Res Methodol 9, 57 (2009). https://doi.org/10.1186/1471-2288-9-57
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tipping_point_model_based.R
\name{tipping_point_model_based}
\alias{tipping_point_model_based}
\title{Tipping Point Analysis (Model-Based)}
\usage{
tipping_point_model_based(
  dat,
  reason,
  impute,
  imputation_model = "weibull",
  J = 10,
  tipping_range = seq(0.05, 1, by = 0.05),
  cox_fit = NULL,
  verbose = FALSE,
  seed = NULL
)
}
\arguments{
\item{dat}{data.frame containing at least 5 columns: TRT01P (treatment arm as factor), AVAL (survival time), EVENT (event indicator), CNSRRS (censoring reason) and MAXAVAL (maximum potential survival time, duration between randomization to data cut-off)}

\item{reason}{Vector specifying censoring reasons to be imputed.}

\item{impute}{a character vector specifying the arm(s) to impute. Can be one arm or both arms (a length-2 vector). Each value must be one of the arms from variable TRT01P. When both arms are supplied, imputation is applied independently.}

\item{imputation_model}{used to fit model to observed data (should be "Weibull" or "exponential")}

\item{J}{numeric indicating number of imputations.}

\item{tipping_range}{Numeric vector, or when \code{length(impute) == 2} optionally a named or unnamed list of two numeric vectors (one per arm). When a list is supplied, all combinations of the two vectors are evaluated.}

\item{cox_fit}{A Cox model that will be used to calculate HRs on imputed datasets.
In case of inclusion of stratification factors or covariates, conditional HR will be used.}

\item{verbose}{Logical. If \code{TRUE}, prints progress and analysis details.}

\item{seed}{Integer, default as NULL. Random seed for reproducibility.}
}
\value{
A \code{tipse} object containing:
\describe{
\item{original data}{Input argument from 'data'.}
\item{imputation_results}{A data frame of combined pooled model results across tipping points}
\item{original_HR}{The original hazard ratio.}
\item{reason_to_impute}{Input argument from 'reason'.}
\item{arm_to_impute}{Input argument from 'impute'.}
\item{method_to_impute}{Input argument from 'method'.}
\item{imputation_data}{A list of imputed datasets for each tipping point value.}
\item{seed}{Random seed.}
}
}
\description{
Performs a model-based tipping point analysis on time-to-event data by
repeatedly imputing censored observations under varying assumptions. The model-based
framework assumes that censored patients have a multiple of hazard fitted via a parametric survival model
compared to the rest of patients in the same arm (Akinson et al, 2019).
}
\details{
The \strong{model-based tipping point analysis} provides a reproducible and intuitive
framework for exploring the robustness of treatment effects in time-to-event
(survival) endpoints when censoring may differ between study arms.

A parametric survival model is fitted using maximum likelihood.
This function applies a hazard deflation on control arm or hazard inflation on treatment
arm, and impute survival times based on the parametric model with additional sampling of the parameters from a multivariate normal distribution.
This imputation procedure is iterated across a range of
tipping point parameters \code{tipping_range}. For each parameter value:
\enumerate{
\item Multiple imputed datasets are generated (\code{J} replicates), where censored
observations in the selected arm are reassigned
event times according to the imputation method.
\item A Cox proportional hazards model is fitted to each imputed dataset.
\item Model estimates are pooled using \strong{Rubin’s rules} to obtain a combined
hazard ratio and confidence interval for that tipping point parameter.
}

The process yields a series of results showing how the treatment effect changes
as increasingly conservative or optimistic assumptions are made about censored
observations. The \emph{tipping point} is defined as the smallest value (hazard inflation)
or biggest value (hazard deflation) of the sensitivity parameter for which the upper
bound of the hazard ratio confidence interval crosses 1 - i.e., where the
apparent treatment benefit is lost.
}
\examples{
cox1 <- survival::coxph(Surv(AVAL, EVENT) ~ TRT01P, data = codebreak200)
result <- tipping_point_model_based(
  dat = codebreak200,
  reason = "Early dropout",
  impute = "docetaxel",
  imputation_model = "weibull",
  J = 10,
  tipping_range = seq(0.1, 1, by = 0.05),
  cox_fit = cox1
)
}
\references{
Atkinson, A., Kenward, M. G., Clayton, T., & Carpenter, J. R. (2019).
Reference‐based sensitivity analysis for time‐to‐event data. Pharmaceutical statistics, 18(6), 645-658.
}
